Package: EHRtemporalVariability
Type: Package
Title: Delineating Temporal Dataset Shifts in Electronic Health Records
Version: 1.2.2
Date: 2026-04-27
Encoding: UTF-8
Description: Functions to delineate temporal dataset shifts in Electronic Health 
             Records through the projection and visualization of dissimilarities 
             among data temporal batches. This is done through the estimation of 
             data statistical distributions over time and their projection in 
             non-parametric statistical manifolds, uncovering the patterns of the 
             data latent temporal variability. 'EHRtemporalVariability' is 
             particularly suitable for multi-modal data and categorical variables 
             with a high number of values, common features of biomedical data where 
             traditional statistical process control or time-series methods may not 
             be appropriate. 'EHRtemporalVariability' allows you to explore and 
             identify dataset shifts through visual analytics formats such as 
             Data Temporal heatmaps and Information Geometric Temporal (IGT) plots. 
             An additional 'EHRtemporalVariability' Shiny app can be used to load 
             and explore the package results and even to allow the use of these 
             functions to those users non-experienced in R coding. (Sáez et al. 2020)
             <doi:10.1093/gigascience/giaa079>.
Depends: R (>= 4.3.0), dplyr
License: Apache License 2.0 | file LICENSE
Imports: plotly, zoo, xts, lubridate, RColorBrewer, viridis, scales,
        methods, MASS
Suggests: knitr, rmarkdown, devtools, dbscan, webshot, httr
VignetteBuilder: knitr
NeedsCompilation: no
Authors@R: c(
  person("Carlos", "Sáez", email = "carsaesi@upv.es", role = c("aut", "cre")),
  person("Alba", "Gutiérrez-Sacristán", email = "Alba_Gutierrez@hms.harvard.edu", role = c("aut")),
  person("Isaac", "Kohane", email = "Isaac_Kohane@hms.harvard.edu", role = c("aut")),
  person("Juan M", "García-Gómez", email = "juanmig@upv.es", role = c("aut")),
  person("Paul", "Avillach", email = "Paul_Avillach@hms.harvard.edu", role = c("aut")),
  person("Biomedical Data Science Lab, Universitat Politècnica de València (Spain)", role = c("cph")),
  person("Department of Biomedical Informatics, Harvard Medical School", role = c("cph"))
  )
Maintainer: Carlos Sáez <carsaesi@upv.es>
RoxygenNote: 7.3.3
URL: https://github.com/hms-dbmi/EHRtemporalVariability
Packaged: 2026-04-28 07:36:08 UTC; Carlos
Author: Carlos Sáez [aut, cre],
  Alba Gutiérrez-Sacristán [aut],
  Isaac Kohane [aut],
  Juan M García-Gómez [aut],
  Paul Avillach [aut],
  Biomedical Data Science Lab, Universitat Politècnica de València
    (Spain) [cph],
  Department of Biomedical Informatics, Harvard Medical School [cph]
Repository: CRAN
Date/Publication: 2026-04-28 08:10:17 UTC
Built: R 4.5.2; ; 2026-04-28 10:56:15 UTC; unix

Package: EHRtemporalVariability
Type: Package
Title: Delineating Temporal Dataset Shifts in Electronic Health Records
Version: 1.2.2
Date: 2026-04-27
Encoding: UTF-8
Description: Functions to delineate temporal dataset shifts in Electronic Health 
             Records through the projection and visualization of dissimilarities 
             among data temporal batches. This is done through the estimation of 
             data statistical distributions over time and their projection in 
             non-parametric statistical manifolds, uncovering the patterns of the 
             data latent temporal variability. 'EHRtemporalVariability' is 
             particularly suitable for multi-modal data and categorical variables 
             with a high number of values, common features of biomedical data where 
             traditional statistical process control or time-series methods may not 
             be appropriate. 'EHRtemporalVariability' allows you to explore and 
             identify dataset shifts through visual analytics formats such as 
             Data Temporal heatmaps and Information Geometric Temporal (IGT) plots. 
             An additional 'EHRtemporalVariability' Shiny app can be used to load 
             and explore the package results and even to allow the use of these 
             functions to those users non-experienced in R coding. (Sáez et al. 2020)
             <doi:10.1093/gigascience/giaa079>.
Depends: R (>= 4.3.0), dplyr
License: Apache License 2.0 | file LICENSE
Imports: plotly, zoo, xts, lubridate, RColorBrewer, viridis, scales,
        methods, MASS
Suggests: knitr, rmarkdown, devtools, dbscan, webshot, httr
VignetteBuilder: knitr
NeedsCompilation: no
Authors@R: c(
  person("Carlos", "Sáez", email = "carsaesi@upv.es", role = c("aut", "cre")),
  person("Alba", "Gutiérrez-Sacristán", email = "Alba_Gutierrez@hms.harvard.edu", role = c("aut")),
  person("Isaac", "Kohane", email = "Isaac_Kohane@hms.harvard.edu", role = c("aut")),
  person("Juan M", "García-Gómez", email = "juanmig@upv.es", role = c("aut")),
  person("Paul", "Avillach", email = "Paul_Avillach@hms.harvard.edu", role = c("aut")),
  person("Biomedical Data Science Lab, Universitat Politècnica de València (Spain)", role = c("cph")),
  person("Department of Biomedical Informatics, Harvard Medical School", role = c("cph"))
  )
Maintainer: Carlos Sáez <carsaesi@upv.es>
RoxygenNote: 7.3.3
URL: https://github.com/hms-dbmi/EHRtemporalVariability
Packaged: 2026-04-28 07:36:08 UTC; Carlos
Author: Carlos Sáez [aut, cre],
  Alba Gutiérrez-Sacristán [aut],
  Isaac Kohane [aut],
  Juan M García-Gómez [aut],
  Paul Avillach [aut],
  Biomedical Data Science Lab, Universitat Politècnica de València
    (Spain) [cph],
  Department of Biomedical Informatics, Harvard Medical School [cph]
Repository: CRAN
Date/Publication: 2026-04-28 08:10:17 UTC
Built: R 4.6.0; ; 2026-04-28 11:14:56 UTC; unix

# Generated by roxygen2: do not edit by hand

export(estimateDataTemporalMap)
export(estimateIGTTrajectory)
export(formatDate)
export(icd9toPheWAS)
exportClasses(DataTemporalMap)
exportMethods(estimateIGTProjection)
exportMethods(plotDataTemporalMap)
exportMethods(plotIGTProjection)
exportMethods(trimDataTemporalMap)
import(dplyr)
importFrom(methods,.valueClassTest)
importFrom(methods,new)
importFrom(stats,complete.cases)
importFrom(stats,setNames)
importFrom(utils,read.csv)
importFrom(utils,read.delim)

# EHRtemporalVariability [![](https://www.r-pkg.org/badges/version/EHRtemporalVariability)](https://cran.r-project.org/package=EHRtemporalVariability) [![](https://cranlogs.r-pkg.org/badges/grand-total/EHRtemporalVariability)](https://cranlogs.r-pkg.org/badges/grand-total/EHRtemporalVariability)

R package for delineating temporal dataset shifts in Electronic Health Records

## What is this repository for?
Functions to delineate temporal dataset shifts in Electronic Health Records through the projection and visualization of dissimilarities among data temporal batches. This is done through the estimation of data statistical distributions over time and their projection in non-parametric statistical manifolds, uncovering the patterns of the data latent temporal variability. Dataset shifts can be explored and identified through visual analytics formats such as Data Temporal Heatmaps (DTHs) and Information Geometric Temporal (IGT) plots [1-3](https://github.com/hms-dbmi/EHRtemporalVariability#Citation). An additional [EHRtemporalVariability Shiny app](https://github.com/hms-dbmi/EHRtemporalVariability-shiny) can be used to load and explore the package results and even to allow the use of these functions to those users non-experienced in R coding.

![](./vignettes/dth_igtplot.png)
*Sample DTH and IGT plot of variable 'Diagnosis Code #1 - PheWAS Code' of the [NHDS](https://www.cdc.gov/nchs/nhds/) data*

## Background
When making data science tasks on a longtime dataset, one must be aware that changes of reference and nature-induced changes in the data acquisition context can occur. These changes will likely be reflected as changes in the statistical distributions of data in form of dataset shifts. Temporal variability artifacts can introduce undesired heterogeneity in data over time, what can potentially hinder data quality and challenge the secondary use of data, particularly for population and data-driven research, as well as machine-learning. Statistical process control or time-series approaches can help detecting changes in summary statistics of data. However, there is a risk for loss of information, especially when deployed when using categorical variables with a particularly high number of values, as well as in multimodal statistical distributions in which multiple sub-phenotypes are present. EHRtemporalVariability provides the means to visually and analytically delineate dataset shifts in multi-modal and highly coded information, with no distributional assumptions made.

Our method is based upon the estimation and comparison of data statistical distributions over time [1-3](https://github.com/hms-dbmi/EHRtemporalVariability#Citation). DTHs allow users to explore changes in absolute and relative frequencies over time—and at multiple variable values simultaneously (e.g., frequencies of phenotypes). IGT plots project time batches as a series of points. The distances between them correspond to the dissimilarity of their statistical distributions. This yields an empirical layout of temporal relationships between batches, namely a non-parametric temporal statistical manifold. IGT plots allow users to visually identify four types of changes: trends, represented as continuously flowing time batches; abrupt changes, shown as gaps between groups of batches; temporal subgroups, depicted as clusters of batches; and seasonality, portrayed as temporal cycles. Additionally, batches are labeled by date and color-coded to distinguish seasonal effects. For more information on how to use EHRtemporalVariability to delinate and identify these changes, please see the [EHRtemporalVariability vignette](https://personales.upv.es/carsaesi/EHRtemporalVariability/EHRtemporalVariability.html)

## Package' Status

 * __Version__: 1.2.2
 * __Authors__: Carlos Sáez (UPV-HMS), Alba Gutiérrez-Sacristán (HMS), Isaac Kohane (HMS), Juan M García-Gómez (UPV), Paul Avillach (HMS)
 * __Maintainer__: Carlos Sáez (UPV-HMS)
 
 Copyright: 2019 - Biomedical Data Science Lab, Universitat Politècnica de València, Spain (UPV) - Department of Biomedical Informatics, Harvard Medical School (HMS)

## Documentation

* Vignette: [EHRtemporalVariability: Delineating temporal dataset shifts in electronic health records](https://personales.upv.es/carsaesi/EHRtemporalVariability/EHRtemporalVariability.html)

* [Package help](https://github.com/hms-dbmi/EHRtemporalVariability/raw/master/vignettes/EHRtemporalVariabilityHelp.pdf)

## Citation

If you use EHRtemporalVariability, please cite:

Sáez C, Gutiérrez-Sacristán A, Kohane I, García-Gómez JM, Avillach P. EHRtemporalVariability: delineating temporal data-set shifts in electronic health records. GigaScience, Volume 9, Issue 8, August 2020, giaa079. https://doi.org/10.1093/gigascience/giaa079

Consider also citing any of the original methods and case studies describing the approach:

[1]: Sáez C, Rodrigues PP, Gama J, Robles M, García-Gómez JM. Probabilistic change detection and visualization methods for the assessment of temporal stability in biomedical data quality. Data Mining and Knowledge Discovery. 2015;29:950–75. https://doi.org/10.1007/s10618-014-0378-6

[2]: Sáez C, Zurriaga O, Pérez-Panadés J, Melchor I, Robles M, García-Gómez JM. Applying probabilistic temporal and multisite data quality control methods to a public health mortality registry in spain: A systematic approach to quality control of repositories. Journal of the American Medical Informatics Association. 2016;23:1085–95. https://doi.org/10.1093/jamia/ocw010

[3]: Sáez C, García-Gómez JM. Kinematics of Big Biomedical Data to characterize temporal variability and seasonality of data repositories: Functional Data Analysis of data temporal evolution over non-parametric statistical manifolds. International Journal of Medical Informatics. 2018;119:109–24. https://doi.org/10.1016/j.ijmedinf.2018.09.015


## Download

Install the latest released version from CRAN

```R
install.packages("EHRtemporalVariability")
```

Download the latest development code of EHRtemporalVariability from GitHub using [devtools](https://cran.r-project.org/package=devtools) with

```R
devtools::install_github("hms-dbmi/EHRtemporalVariability")
```

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
    echo = TRUE,
    warning=FALSE,
    fig.width = 7,
    fig.height = 4,
    out.width = "100%")

## ----cran, message=FALSE, eval=FALSE, warning=FALSE---------------------------
# install.packages("EHRtemporalVariability")
# library(EHRtemporalVariability)

## ----crantrue, echo=FALSE, message=FALSE, eval=TRUE, warning=FALSE------------
library(EHRtemporalVariability)

## ----devtls, message=FALSE, eval=FALSE, warning=FALSE-------------------------
# install.packages("devtools")
# library(devtools)

## ----bioC, message=FALSE, eval=FALSE, warning=FALSE---------------------------
# install_github("hms-dbmi/EHRtemporalVariability")
# library( EHRtemporalVariability )

## ----EHRtemporalVariabilityObj1, echo = FALSE, eval = TRUE, warning=FALSE-----
githubURL <- "https://github.com/hms-dbmi/EHRtemporalVariability-DataExamples/raw/master/variabilityDemoNHDS.RData"
if (httr::http_error(githubURL)) {
      message("URL not accessible or no internet connection.")
      load(system.file("extdata","variabilityDemoNHDSdiagcode1-phewascode.RData",package="EHRtemporalVariability"))
} else {
    load(url(githubURL))
}

## ----variabilityObj2, eval=TRUE, warning=FALSE--------------------------------
class( probMaps$`diagcode1-phewascode` )

## ----igtProjObj, eval=TRUE, warning=FALSE-------------------------------------
class( igtProjs$`diagcode1-phewascode` )

## ----readCSV, eval=TRUE, warning=FALSE----------------------------------------
dataset <- read.csv2( "http://github.com/hms-dbmi/EHRtemporalVariability-DataExamples/raw/master/nhdsSubset.csv", 
                      sep  = ",",
                      header = TRUE, 
                      na.strings = "", 
                      colClasses = c( "character", "numeric", "factor",
                                      "numeric" , rep( "factor", 22 ) ) )
head( dataset)

## ----formatDate2, eval=TRUE, warning=FALSE------------------------------------
class( dataset$date )
datasetFormatted <- EHRtemporalVariability::formatDate(
              input         = dataset,
              dateColumn    = "date",
              dateFormat = "%y/%m"
             )
head( datasetFormatted )[1:5, 1:5]
class( datasetFormatted$date )

## ----icd9toPheWAS, eval=TRUE, message=FALSE, warning=FALSE--------------------
datasetPheWAS <- icd9toPheWAS(data           = datasetFormatted,
                              icd9ColumnName = "diagcode1",
                              phecodeDescription = TRUE,
                              missingValues  = "N/A", 
                              statistics     = TRUE, 
                              replaceColumn  = FALSE)

head( datasetPheWAS[, c( "diagcode1", "diagcode1-phewascode")] )

## ----estimateDataTemporalMap, eval=FALSE, warning=FALSE-----------------------
# probMaps <- estimateDataTemporalMap(data           = datasetPheWAS,
#                                     dateColumnName = "date",
#                                     period         = "month")

## ----estimateDataTemporalMapOutput, eval=TRUE, warning=FALSE------------------
class( probMaps )
class( probMaps[[ 1 ]] )

## ----estimateDataTemporalMapSupport, eval=FALSE, warning=FALSE----------------
# supports <- vector("list",2)
# names(supports) <- c("age","diagcode1")
# supports[[1]] <- 1:18
# supports[[2]] <- c("V3000","042--","07999","1550-","2252-")
# probMapsWithSupports <- estimateDataTemporalMap(data           = datasetPheWAS,
#                                     dateColumnName = "date",
#                                     period         = "month",
#                                     supports       = supports)

## ----trimDataTemporalMap, eval=TRUE, warning=FALSE----------------------------
class( probMaps[[1]] )
probMapTrimmed <- trimDataTemporalMap( 
                        dataTemporalMap = probMaps[[1]],
                        startDate       = "2005-01-01",
                        endDate         = "2008-12-01"
                                      )
class( probMapTrimmed )

## ----estimateIGTProjection, eval=TRUE, warning=FALSE--------------------------
igtProj <- estimateIGTProjection( dataTemporalMap = probMaps[[1]], 
                                  dimensions      = 2, 
                                  startDate       = "2000-01-01", 
                                  endDate         = "2010-12-31")

## ----estimateIGTProjectionOutput, eval=TRUE, warning=FALSE--------------------
class( igtProj )

## ----sapplyestimateIGTProjection, eval=FALSE, warning=FALSE-------------------
# igtProjs <- sapply ( probMaps, estimateIGTProjection )
# names( igtProjs ) <- names( probMaps )

## ----loadExampleFile, eval=TRUE, warning=FALSE--------------------------------
githubURL <- "https://github.com/hms-dbmi/EHRtemporalVariability-DataExamples/raw/master/variabilityDemoNHDS.RData"
load(url(githubURL))

## ----plotHeatmap, eval=TRUE, warning=FALSE------------------------------------
plotDataTemporalMap(
    dataTemporalMap =  probMaps[["diagcode1-phewascode"]],
    startValue = 2,
    endValue = 20,
    colorPalette    = "Spectral")

## ----plotSeries, eval=TRUE, warning=FALSE-------------------------------------
plotDataTemporalMap(
    dataTemporalMap =  probMaps[["diagcode1-phewascode"]],
    startValue = 2,
    endValue = 10,
    mode = "series")

## ----plotIGTprojection, eval=TRUE, warning=FALSE------------------------------
plotIGTProjection( 
    igtProjection   =  igtProjs[["diagcode1-phewascode"]],
    colorPalette    = "Spectral", 
    dimensions      = 2)

## ----plotIGTprojectionTrajectory, eval=TRUE, warning=FALSE--------------------
plotIGTProjection( 
    igtProjection   =  igtProjs[["diagcode1-phewascode"]],
    colorPalette    = "Spectral", 
    dimensions      = 2,
    trajectory      = TRUE)

## ----saveRData, eval=FALSE, warning=FALSE-------------------------------------
# names( probMaps )
# names( igtProjs )
# save(probMaps, igtProjs, file = "myExport.RData")

## ----dbscan, message=FALSE, eval=FALSE, warning=FALSE-------------------------
# install.packages("dbscan")
# library(dbscan)

## ----dbscantrue, echo=FALSE, message=FALSE, eval=requireNamespace("dbscan", quietly=TRUE), warning=FALSE----
library(dbscan)

## ----temporalSubgroupsClustering, eval=requireNamespace("dbscan", quietly=TRUE), warning=FALSE----
# We set the minimum number of batches in a subgroup as 2 
# We set eps based on the knee of the following KNNdistplot, at around 0.023
# kNNdistplot(igtProj@projection, k = 2, all = FALSE)
igtProj = igtProjs[["diagcode1-phewascode"]]
# We select the 2 first dimensions for consistency with the IGT plot examples above
dbscanResults <- dbscan::dbscan(igtProj@projection[,c(1,2)], eps = 0.023, minPts = 2)
clusterNames  <- vector(mode = "character", length = 10)
clusterNames[dbscanResults$cluster == 0] <- "Outlier batches"
clusterNames[! dbscanResults$cluster == 0] <- paste("Temporal subgroup",dbscanResults$cluster[! dbscanResults$cluster == 0])
plotly::plot_ly(x = igtProj@projection[,1], y = igtProj@projection[,2],
              color = as.factor(clusterNames),
              type = "scatter", mode = "markers",
              text = paste0("Date: ",igtProj@dataTemporalMap@dates)) %>%
              plotly::config(displaylogo = FALSE)

# Copyright 2019 Biomedical Data Science Lab, Universitat Politècnica de València (Spain) - Department of Biomedical Informatics, Harvard Medical School (US)
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
# http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

jsdiv <- function(p, q){
  # m <- log2(0.5 * (p + q))
  # jsdiv <- 0.5 * (sum(p * (log2(p) - m),na.rm = TRUE) + sum(q * (log2(q) - m), na.rm = TRUE))
    m <- 0.5 * (p + q)
    jsdiv <- 0.5 * (sum(p * (log2(p/m)), na.rm = TRUE) + sum(q * (log2(q/m)), na.rm = TRUE))
    return(jsdiv)
}

# Copyright 2019 Biomedical Data Science Lab, Universitat Politècnica de València (Spain) - Department of Biomedical Informatics, Harvard Medical School (US)
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
# http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' Function to transform dates into "Date" R format
#'
#' Given a \code{data.frame} object with a column of dates in 'character' format, 
#' it generates a new \code{data.frame} object with the dates transformed into 
#' "Date" R format. 
#'
#' @param input A \code{data.frame} object with at least one column of dates.
#' @param dateColumn The name of the column containing the date.
#' @param dateFormat By default \code{'\%y/\%m/\%d'}. Change it to the specific structure of your date format.  
#' @param verbose By default \code{FALSE}. Change it to \code{TRUE} to get an on-time log from the function.
#' @return An object of class \code{data.frame} with the date column transform into 'Date' R class. 
#' @examples
#' 
#' dataset <- read.csv2(xzfile(system.file("extdata",
#'                                    "nhdsSubset.csv.xz",
#'                                    package="EHRtemporalVariability")), 
#'                      sep  = ",",
#'                      header = TRUE, 
#'                      na.strings = "", 
#'                      colClasses = c( "character", "numeric", "factor",
#'                                      "numeric" , rep( "factor", 22 ) ) )
#'                      
#' datasetFormatted <- formatDate( 
#'               input         = dataset, 
#'               dateColumn    = "date", 
#'               dateFormat    = "%y/%m",
#'              )
#' @export formatDate
#' @importFrom stats complete.cases setNames
formatDate <- function( input, dateColumn, dateFormat = "%y/%m/%d", verbose = FALSE ){
   
    
    if( dateColumn %in% colnames( input ) ){
        colNum <- which( colnames( input ) == dateColumn )
    }else{
        message( paste0( "There is no column in your data.frame named as: ", dateColumn ))
        stop()
    }
    
    if( inherits( input[, colNum], "Date" )){
        output <- input
    }else{
        
        if( verbose == TRUE){
        message( paste0( "Formatting the ", dateColumn, " column" ) )
        }
        
    is_letter <- function(x, pattern=c(letters, LETTERS)){
            sapply(x, function(y){
                any(sapply(pattern, function(z) grepl(z, y, fixed=T)))
            })}
    
    if( is_letter( dateFormat, c("Y", "y") ) == TRUE &
            is_letter( dateFormat, c("m", "M", "b", "B","h") ) == TRUE &
            is_letter( dateFormat, c("d", "D" ) ) == TRUE ){
        
        if(verbose == TRUE){
            message("The data format contains year, month and day")
        }
            input$dateFormat <- as.Date( input[ , colNum ], dateFormat )
        }else if( is_letter( dateFormat, c("Y", "y") ) == TRUE &
                  is_letter( dateFormat, c("m", "M", "b", "B","h") ) == TRUE)
                  {
            if(verbose == TRUE){
            message("The data format contains year and month but not day")
            message("Take into account that if you perform an analysis by 
                    week, the day will be automatically assigned as the 
                    first day of the month.")
            }
            input$dateFormat <- zoo::as.Date( zoo::as.yearmon( input[, colNum ], dateFormat ) )
        }else if( is_letter( dateFormat, c("Y", "y") ) == TRUE )
        {
            if(verbose == TRUE){
            message("The data format contains only the year")
            message("Take into account that if you perform an analysis by 
                    week or by month, they will be automatically assigned as the 
                    first day of the month and first month of the year.")
            }
            input$dateFormat <-as.Date( input[, colNum ], dateFormat  )
        }else{
            message("Please, check the format of the date. At least it should
                    contain the year. ")
            stop()
        }
    
    #check if the date transformation is ok 
    checkingComplete <- input[complete.cases(input[ , "dateFormat" ] ) , ]
    if( nrow( checkingComplete ) == nrow( input ) ){
        input[, colNum ] <- input$dateFormat
    }else{
        checkingComplete$date <- checkingComplete$dateFormat
        output <- checkingComplete[, -which( names( checkingComplete ) == "dateFormat" ) ]
        message( paste0( "There are ", nrow( input ) - nrow( checkingComplete ), " rows 
                         that do not contain date information. They have been removed.") )
    }
    
    
    output <- input[, ! names( input) == "dateFormat" ]
    }
    return(output)
}

# Copyright 2019 Biomedical Data Science Lab, Universitat Politècnica de València (Spain) - Department of Biomedical Informatics, Harvard Medical School (US)
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
# http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

checkDataTemporalMap <- function(object) {
    errors <- character()
    if (!all(dim(object@probabilityMap)==dim(object@countsMap))) {
        msg <- "the dimensions of probabilityMap and countsMap do not match"
        errors <- c(errors, msg)
    }
    if (nrow(object@probabilityMap) != length(object@dates)){
        msg <- paste("the length of dates must match the rows of probabilityMap and countsMap")
        errors <- c(errors, msg)
    }
    if (!is.null(object@support) && ncol(object@probabilityMap) != nrow(object@support)){
        msg <- paste("the length of support must match the columns of probabilityMap and countsMap")
        errors <- c(errors, msg)
    }
    validPeriods <- c('week', 'month', 'year')
    if( !object@period %in% validPeriods ){
        msg <- paste("period must be one of the following:", paste(validPeriods,collapse = ', '))
        errors <- c(errors, msg)
    }
    validTypes <- c('numeric', 'integer', 'character', 'factor', 'Date')
    if( !object@variableType %in% validTypes ){
        msg <- paste("variableType must be one of the following:",paste(validTypes,collapse = ', '))
        errors <- c(errors, msg)
    }
    if (length(errors) == 0) TRUE else errors
}

#' Class DataTemporalMap
#'
#' Class \code{DataTemporalMap} object contains the statistical distributions of data estimated at a 
#' specific time period. Both relative and absolute frequencies are included. 
#'
#' Objects of this class are generated automatically by the \code{estimateDataTemporalMap} function, 
#' but its construction and extension is open towards fostering its use through external methods. 
#' E.g., one may use additional probability distribution estimation methods, or even construct 
#' compatible DataTemporalMaps for other unstructured data such as images or free text.
#'
#' @name DataTemporalMap-class
#' @rdname DataTemporalMap-class
#' @aliases DataTemporalMap,DataTemporalMap-class
#' @slot probabilityMap v-by-d numerical \code{matrix} representing the probability distribution 
#' temporal map (relative frequency).
#' @slot countsMap v-by-d numerical \code{matrix} representing the counts temporal map 
#' (absolute frequency).
#' @slot dates d-dimensional \code{Date} array of the temporal batches.
#' @slot support v-by-1 numerical or character \code{matrix} representing the support 
#' (the value at each bin) of probabilityMap and countsMap.
#' @slot variableName name of the variable (character).
#' @slot variableType type of the variable (character) among "numeric", "character", "Date" and "factor".
#' @slot period batching period among "week", "month" and "year".
#' @return A \code{DataTemporalMap} object.
#' @examples
#' 
#' # Generation through estimateDataTemporalMap function:
#' dataset <- read.csv2(xzfile(system.file("extdata",
#'                                    "nhdsSubset.csv.xz",
#'                                    package="EHRtemporalVariability")), 
#'                      sep  = ",",
#'                      header = TRUE, 
#'                      na.strings = "", 
#'                      colClasses = c( "character", "numeric", "factor",
#'                                      "numeric" , rep( "factor", 22 ) ) )
#' 
#' datasetFormatted <- EHRtemporalVariability::formatDate(
#'                      input         = dataset,
#'                      dateColumn    = "date",
#'                      dateFormat = "%y/%m")
#' 
#' probMaps <- estimateDataTemporalMap(data = datasetFormatted, 
#'                      dateColumnName = "date", 
#'                      period         = "month")
#' 
#' class( probMaps[[1]] ) 
#' 
#' # Manual generation:
#' countsMatrix <- matrix(sample.int(25, size = 12*10, replace = TRUE), nrow = 12, ncol = 10)
#' probabilityMatrix <- sweep(countsMatrix,1,rowSums(countsMatrix),"/")
#' dates <- seq(Sys.Date(),(Sys.Date()+30*11),30)
#' x <- new('DataTemporalMap', probabilityMap = probabilityMatrix, 
#'                      countsMap = countsMatrix, dates = dates, support = data.frame(1:10), 
#'                      variableName = "example", variableType = "numeric", period = "month")
#' plotDataTemporalMap(x)
#' 
#' @exportClass DataTemporalMap
DataTemporalMap <- setClass( "DataTemporalMap",
         slots =
             c( 
                 probabilityMap = "matrix",     # d-by-v matrix representing the probability distribution temporal map
                 countsMap      = "matrix",     # d-by-v matrix representing the absolute frequencies temporal map
                 dates          = "Date",       # d-dimensional date array of the temporal batches
                 support        = "data.frame", # v-by-1 matrix representing the support bins of probabilityMap and countsMap 
                 variableName   = "character",  # name of the variable
                 variableType   = "character",  # type of the variable (numeric, character, Date, factor)
                 period         = "character"   # batching period (week, month, year)
             ),
         prototype = 
             list(
                 probabilityMap  = NULL,
                 countsMap       = NULL,
                 dates           = NULL,
                 support         = NULL,
                 variableName    = NULL,
                 variableType    = NULL,
                 period          = NULL
             ),
         validity = checkDataTemporalMap
)

#' Class IGTProjection
#'
#' Class \code{IGTProjection} object contains the estimated Information Geometric Temporal plot projection
#' of a \code{DataTemporalMap} object.
#'
#' Objects of this class are generated automatically by the \code{estimateIGTProjection} function, and can
#' be passed to the \code{plotIGTProjection} function for plotting the Information Geometric Temporal plot.
#'
#' @name IGTProjection-class
#' @rdname IGTProjection-class
#' @aliases IGTProjection,IGTProjection-class
#' @slot dataTemporalMap the \code{DataTemporalMap} object from which the projection was estimated
#' @slot projection d-by-c \code{matrix} of the IGT projection for d temporal batches in c dimensions calculated from the \code{estimateIGTProjection} function
#' @slot embeddingType embedding function used for the IGT projection calculation in the \code{estimateIGTProjection} function
#' @slot stress stress achieved in the lower-dimensional embedding, depending on the embedding type (e.g., classicalmds will return 1-GOF as returned by stats::cmdscale function, and nonmetricmds will return final stress in percent, as returned by the MASS::isoMDS function)
#' @return A \code{IGTProjection} object.
#' 
IGTProjection <- setClass( "IGTProjection",
           slots =
               c( 
                   dataTemporalMap = "DataTemporalMap", # v-by-d matrix representing the data temporal map
                   projection      = "matrix",          # d-by-c matrix of the IGT projection for d temporal batches in c dimensions
                   embeddingType   = "character",       # embedding function used for the IGT projection calculation, classical multidimensional scaling by default
                   stress          = "numeric"           # stress achieved in the lower-dimensional embedding, depending on the embedding type (e.g., classicalmds will return 1-GOF as returned by stats::cmdscale function, and nonmetricmds will return final stress in percent, as returned by the MASS::isoMDS function)
               ),
           prototype = 
               list(
                   dataTemporalMap = NULL,
                   projection      = NULL,
                   embeddingType   = NULL,
                   stress          = NULL
               )
)

library(EHRtemporalVariability)

dataset <- read.csv2( "http://github.com/hms-dbmi/EHRtemporalVariability-DataExamples/raw/master/nhdsSubset.csv", 
                      sep  = ",",
                      header = TRUE, 
                      na.strings = "", 
                      colClasses = c( "character", "numeric", "factor",
                                      "numeric" , rep( "factor", 22 ) ) )

datasetFormatted <- EHRtemporalVariability::formatDate(
    input         = dataset,
    dateColumn    = "date",
    dateFormat = "%y/%m"
)

datasetPheWAS <- icd9toPheWAS(data           = datasetFormatted,
                              icd9ColumnName = "diagcode1",
                              phecodeDescription = TRUE,
                              missingValues  = "N/A", 
                              statistics     = TRUE, 
                              replaceColumn  = FALSE)

probMaps <- estimateDataTemporalMap(data           = datasetPheWAS, 
                                    dateColumnName = "date", 
                                    period         = "month")

igtProjs <- sapply ( probMaps, estimateIGTProjection )
names( igtProjs ) <- names( probMaps )

save(probMaps, igtProjs, file = "myExport.RData")

# Copyright 2019 Biomedical Data Science Lab, Universitat Politècnica de València (Spain) - Department of Biomedical Informatics, Harvard Medical School (US)
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
# http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' Function to transform ICD9-CM codification into PheWAS code
#'
#' Given a  \code{data.frame} object with a column of ICD9-CM codes, it generates a 
#' new \code{data.frame} object with the ICD9-CM codes transformed into PheWAS codes. 
#'
#' @param data A \code{data.frame} object with at least one column of ICD9-CM codes that
#' one to be transformed into a PheWAS code. 
#' @param icd9ColumnName The name of the column containing the ICD9-CM.
#' @param missingValues The value used to determine missing values in the data.frame. 
#' @param phecodeDescription By default \code{FALSE}. Change it to \code{TRUE} to map
#' to the PheWAS code description instead to the PheWAS numeric code. 
#' @param statistics By default \code{FALSE}. Change it to \code{TRUE} to show the 
#' summary of the mapping like the percentage of initial ICD9-CM codes mapped to PheWAS code. 
#' @param replaceColumn By default \code{TRUE}. Change it to \code{FALSE} in order to create a
#' new column with the PheWAS code maintaining the ICD9-CM code.  
#' @param verbose By default \code{FALSE}. Change it to \code{TRUE} to get an
#' on-time log from the function.
#' @return An object of class \code{data.frame} with the ICD9-CM column transform into
#' PheWAS codes. 
#' @examples
#' dataset <- read.csv2(xzfile(system.file("extdata",
#'                                    "nhdsSubset.csv.xz",
#'                                    package="EHRtemporalVariability")), 
#'                      sep  = ",",
#'                      header = TRUE, 
#'                      na.strings = "", 
#'                      colClasses = c( "character", "numeric", "factor",
#'                                      "numeric" , rep( "factor", 22 ) ) )
#' 
#' datasetPheWAS <- icd9toPheWAS( data           = dataset,
#'                               icd9ColumnName  = "diagcode1", 
#'                               missingValues   = "N/A", 
#'                               replaceColumn   = TRUE, 
#'                               statistics      = TRUE 
#'                               )
#' @export icd9toPheWAS
#' @import dplyr 
#' @importFrom utils read.delim
#' @importFrom utils read.csv
icd9toPheWAS <- function( data, icd9ColumnName, missingValues = "NA", phecodeDescription = FALSE, statistics = FALSE, replaceColumn = TRUE, verbose = FALSE ){
    
    if( verbose == TRUE){
        message("Loading the ICD9-CM mapping file") 
    }
    
    #load the icd9 mapping file
    icd9File <- read.delim( file   = xzfile(system.file("extdata",
                                                 "icd9mappingFile.csv.xz",
                                                 package="EHRtemporalVariability")),
                            header     = TRUE,
                            sep        = "\t",
                            colClasses = "character" )

    #check if the column name of the icd9 column exists and save it
    input <- data
    if( icd9ColumnName %in% colnames( input ) ){
        colNum <- which( colnames( input ) == icd9ColumnName )
    }else{
        message( paste0( "There is no column in your data.frame named as: ", icd9ColumnName ))
        stop()
    }
    
    #check if the selected column is factor and if so coerce to character
    if (inherits( input[[icd9ColumnName]] , 'factor')){
        input[[icd9ColumnName]] <- as.character( input[[icd9ColumnName]] )
    }

    if( verbose == TRUE){
        message( paste0( "Checking that the ", icd9ColumnName, " follows the ICD9 structure" ) )
        message( "Missing values will be substituted by 'MissingValues'")
    }
    
    #check if the icd9 column will be replace or another one will be created
    if( replaceColumn == FALSE){
        input$newPhewasColumn <- input[, colNum]
        colNum <- which( colnames( input ) == "newPhewasColumn" )
        colnames( input )[ colNum ] <- paste0( icd9ColumnName, "-phewascode")
    }
    
    #substitute the missing values
    input[, colNum] <- gsub( missingValues, "MissingValues", input[, colNum])

    if( verbose == TRUE){
        message( "Those characters that are non alpha numeric will be removed")
    }
    
    #we remove the non-alphanumeric symbols like "." from the icd9 codes 
    input[, colNum] <- gsub("[^[:alnum:] ]", "", input[, colNum] )

    if( verbose == TRUE){
        message( "Mapping the ICD9-CM to PheWAS code." )
        message( "Please note that Phecode Map 1.2 will be used for this process" )
    }
    
    #create a new data set with the correspondence between icd9 and phewas code   
    colnames( icd9File )[5] <- icd9ColumnName
    completeSet <- dplyr::left_join( input, 
                                     icd9File )

    #determine how many cases are missing values
    naCases      <- which( is.na( completeSet$PheCode ) )
    missingCases <-  naCases[ completeSet[naCases, colNum] %in% "MissingValues"]
    completeSet$PheCode[ missingCases ] <- "MissingValues"
    completeSet$Phenotype[ missingCases ] <- "MissingValues"
    
    #determine how many codes are external causes (not mapped to phewas codes)
    nwNaCases <- setdiff( naCases, missingCases )
    externalCases <- nwNaCases[ substr( as.character(completeSet[nwNaCases, colNum]),1,1 ) == "E" ]
    completeSet$PheCode[ externalCases ] <- "ExternalCauses"
    completeSet$Phenotype[ externalCases ] <- "ExternalCauses"
    
    #determine how many codes are V codes that does not have an assigned icd9 code
    nw2NaCases <- setdiff( nwNaCases, externalCases)
    vCodeNoMap <- nw2NaCases[ substr( as.character(completeSet[nw2NaCases, colNum]),1,1 ) == "V" ]
    completeSet$PheCode[ vCodeNoMap ] <- "VcodeNotMapped"
    completeSet$Phenotype[ vCodeNoMap ] <- "VcodeNotMapped"
    
    #determine how many codes follow an icd9 structure but are not mapped
    nw3NaCases <- setdiff( nw2NaCases, vCodeNoMap )
    icd9Char <- c("V", "E", "1", "2", "3", "4", "5", "6", "7", "8", "9", "0")   
    numbers <- as.character(sprintf('%0.3d', 1:99999)) 
    icd9CodeNoMap <- nw3NaCases[ nchar( completeSet[ nw3NaCases, colNum ] ) <= 5 & 
                                  substr( as.character(completeSet[ nw3NaCases , colNum ]), 1, 1) %in% icd9Char &
                                     substr( as.character(completeSet[ nw3NaCases , colNum ]), 2, nchar(completeSet[ nw3NaCases , colNum ])) %in% numbers ]
    completeSet$PheCode[ icd9CodeNoMap ] <- "ICD9codeNotMapped"
    completeSet$Phenotype[ icd9CodeNoMap ] <- "ICD9codeNotMapped"
    
    #determine how many does not follow the icd9 standard
    nw4NaCases <- setdiff( nw3NaCases, icd9CodeNoMap)
    completeSet$PheCode[ nw4NaCases ] <- "NonICD9code"
    completeSet$Phenotype[ nw4NaCases ] <- "NonICD9code"
    
    #summary of the mapping           
    if( statistics == TRUE){
        
        missingData <- completeSet[ completeSet$PheCode == "MissingValues", ]
        message( paste0( "# Missing values: ", nrow( missingData), " cases ( ", round(nrow( missingData) / nrow( completeSet ) * 100 , 2), "%)"))
        
        externalCausesCodes <- completeSet[ completeSet$PheCode == "ExternalCauses", ]
        message( paste0( "# External causes codes: ", nrow( externalCausesCodes), " cases ( ", round(nrow( externalCausesCodes) / nrow( completeSet ) * 100 , 2), "%)"))
        
        vcodeNotMapped <- completeSet[ completeSet$PheCode == "VcodeNotMapped", ]
        message( paste0( "# V codes not mapped: ", nrow( vcodeNotMapped), " cases ( ", round(nrow( vcodeNotMapped) / nrow( completeSet ) * 100 , 2), "%)"))
        
        icd9InitialCodes <- completeSet[ completeSet$PheCode != "MissingValues"  &
                                         completeSet$PheCode != "ExternalCauses" &
                                         completeSet$PheCode != "VcodeNotMapped" & 
                                         completeSet$PheCode != "ICD9codeNotMapped", ]
        
        message( paste0( "# Total ICD9-CM codes mapped: ", nrow( icd9InitialCodes), " cases ( ", round(nrow( icd9InitialCodes) / nrow( completeSet ) * 100 , 2), "%)"))
        message( paste0( "# Unique ICD9-CM codes mapped: ", length(unique(icd9InitialCodes[, colNum] ) )))
        message( paste0( "# Total PheWAS codes: ", length(unique(icd9InitialCodes$PheCode ) ) ))
        
        
        icd9codeNotMapped <- completeSet[ completeSet$PheCode == "ICD9codeNotMapped", ]
        message( paste0( "# ICD9 codes not mapped: ", nrow( icd9codeNotMapped), " cases ( ", round(nrow( icd9codeNotMapped) / nrow( completeSet ) * 100 , 2), "%)"))
        
        }
    
    #format the final output
    if( phecodeDescription == TRUE ){
        completeSet[, colNum ] <- completeSet$Phenotype
    }else{
        completeSet[, colNum ] <- completeSet$PheCode
    }
    
    finalColumn <- ncol(completeSet)-4 
    completeSet <- as.data.frame( completeSet[ , 1:finalColumn] )
    colnames( completeSet)[ colNum ] <- paste0( icd9ColumnName, "-phewascode")
    
    #if the selected icd9 column was factor, since it was converted to character then change it back to factor
    if (replaceColumn == FALSE & inherits( data[[icd9ColumnName]] , 'factor')){
        completeSet[[icd9ColumnName]] <- factor( completeSet[[icd9ColumnName]] )
    }
    
    return(completeSet)
}

# Copyright 2020 Biomedical Data Science Lab, Universitat Politècnica de València (Spain) - Department of Biomedical Informatics, Harvard Medical School (US)
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
# http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' Estimates a trajectory of the information temporal evolution in a IGT projection by fitting a cubic smoothing spline
#'
#' Estimates a \code{DataTemporalMap} from a \code{data.frame} containing individuals in rows and the 
#' variables in columns, being one of these columns the analysis date (typically the acquisition date). 
#' Will return a \code{DataTemporalMap} object or a \code{list} of \code{DataTemporalMap} objects 
#' depending on the number of analysis variables.
#'
#' @name estimateIGTTrajectory
#' @rdname estimateIGTTrajectory-methods
#' @param igtProjection of class \code{IGTProjection}.
#' @param nPoints the number of points to fit within the IGT projection range. By default 10x the number of time batches, what shows a high resolution trajectory.
#' @return A list containing a \code{data.frame} of the estimated trajectory points, the estimated date for each point, and the fitted trajectory function as \code{smooth.spline} objects.
#' @examples
#' load(system.file("extdata",
#'                  "variabilityDemoNHDSdiagcode1-phewascode.RData",
#'                   package="EHRtemporalVariability"))
#' 
#' igtTrajectory <- estimateIGTTrajectory( igtProjection   =  igtProjs[[1]] )
#' igtTrajectory$points
#' 
#' @export
estimateIGTTrajectory <- function(igtProjection, nPoints = NULL) {

    if( is.null(igtProjection) )
        stop("An input IGT projection object is required.")
    if( is.null(nPoints) )
        nPoints = nrow(igtProjection@projection)*10
    
    nDims    = ncol(igtProjection@projection)
    nBatches = nrow(igtProjection@projection)
    
    t=1:nBatches
    
    tt = seq(1,nBatches,len = nPoints)
    
    trajectoryFunction <- vector(mode = "list")
    points = data.frame(matrix(nrow = nPoints, ncol = nDims))
    names(points) <- sprintf("D%d",1:nDims)
    dates <- seq(min(igtProjection@dataTemporalMap@dates), max(igtProjection@dataTemporalMap@dates), length.out = nPoints)
    
    for (i in 1:nDims) {
        trajectoryFunction[[names(points)[i]]] = stats::smooth.spline(igtProjection@projection[,i])
        points[,i] = stats::predict(trajectoryFunction[[names(points)[i]]], tt)$y
    }
    
    results <- list("points" = points, "dates" = dates, "trajectoryFunction" = trajectoryFunction)
    return(results)
}

# Copyright 2019 Biomedical Data Science Lab, Universitat Politècnica de València (Spain) - Department of Biomedical Informatics, Harvard Medical School (US)
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
# http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' Estimates \code{DataTemporalMap} objects from raw data
#'
#' Estimates a \code{DataTemporalMap} from a \code{data.frame} containing individuals in rows and the 
#' variables in columns, being one of these columns the analysis date (typically the acquisition date). 
#' Will return a \code{DataTemporalMap} object or a \code{list} of \code{DataTemporalMap} objects 
#' depending on the number of analysis variables.
#'
#' @name estimateDataTemporalMap
#' @rdname estimateDataTemporalMap-methods
#' @param data a \code{data.frame} containing as many rows as individuals, and as many columns as the 
#' analysis variables plus the individual acquisition date.
#' @param dateColumnName a string indicating the name of the column in \code{data} containing the 
#' analysis date variable.
#' @param period the period at which to batch data for the analysis from "week", "month" and "year", 
#' with "month" as default.
#' @param startDate a Date object indicating the date at which to start the analysis, in case of being 
#' different from the first chronological date in the date column (the default).
#' @param endDate a Date object indicating the date at which to end the analysis, in case of being 
#' different from the last chronological date in the date column (the default).
#' @param supports a List of objects containing the support of the data distributions for each variable, 
#' in classes \code{numeric}, \code{integer}, \code{character}, or \code{factor} (accordingly to the 
#' variable type), and where the name of the list element must correspond to the column name of its 
#' variable. If not provided it is automatically estimated from data.
#' @param numericVariablesBins the number of bins at which to define the frequency/density histogram 
#' for numerical variables when their support is not provided, 100 as default.
#' @param numericSmoothing a logical value indicating whether a Kernel Density Estimation smoothing 
#' (Gaussian kernel, default bandwidth) is to be applied on numerical variables (the default) or a 
#' traditional histogram instead. See ?density for further details.
#' @param dateGapsSmoothing a logical value indicating whether a linear smoothing is applied to those 
#' time batches without data, by default gaps are filled with NAs.
#' @param verbose By default \code{FALSE}. Change it to \code{TRUE} to get an on-time log from the 
#' function.
#' @return A \code{DataTemporalMap} object.
#' @examples
#' #Load the file 
#' dataset <- read.csv2(xzfile(system.file("extdata",
#'                                    "nhdsSubset.csv.xz",
#'                                    package="EHRtemporalVariability")), 
#'                      sep  = ",",
#'                      header = TRUE, 
#'                      na.strings = "", 
#'                      colClasses = c( "character", "numeric", "factor",
#'                                      "numeric" , rep( "factor", 22 ) ) )
#' #Format the date
#' datasetFormatted <- EHRtemporalVariability::formatDate( input         = dataset,
#'                                             dateColumn    = "date",
#'                                             dateFormat    = "%y/%m")
#' 
#' #Apply the estimateDataTemporalMap
#' probMaps <- estimateDataTemporalMap( data           = datasetFormatted, 
#'                                      dateColumnName = "date", 
#'                                      period         = "month")
#' \dontrun{
#' 
#' For a larger example download the following .csv dataset and continue the steps as above:
#' 
#' gitHubUrl  <- 'http://github.com/'
#' gitHubPath <- 'hms-dbmi/EHRtemporalVariability-DataExamples/'
#' gitHubFile <- 'raw/master/nhdsSubset.csv'
#' inputFile  <-  paste0(gitHubUrl, gitHubPath, gitHubFile)
#' 
#' dataset <- read.csv2( inputFile, 
#'                      sep  = ",",
#'                      header = TRUE, 
#'                      na.strings = "", 
#'                      colClasses = c( "character", "numeric", "factor",
#'                                      "numeric" , rep( "factor", 22 ) ) ) 
#' }
#' @export
estimateDataTemporalMap <- function(data = NULL, dateColumnName = NULL, period = 'month', startDate = NULL, endDate = NULL, supports = NULL, numericVariablesBins = 100, numericSmoothing = TRUE, dateGapsSmoothing = FALSE, verbose = FALSE) {
    
    # Validation of parameters
    if( is.null(data) )
        stop("An input data frame is required.")
    if( ncol(data)<2 )
        stop("An input data frame is required with at least 2 columns, one for dates.")
    if( is.null(dateColumnName) )
        stop("The name of the column including dates is required.")
    if( !dateColumnName %in% colnames(data) )
        stop(paste0('There is not a column named \"',dateColumnName,'\" in the input data.'))
    if( !class(data[[dateColumnName]]) %in% c("Date") )
        stop("The specified date column must be of class Date.")
    validPeriods <- c('week', 'month', 'year')
    if( !period %in% validPeriods )
        stop(paste("Period must be one of the following:", paste(validPeriods,collapse = ', ')))
    validClasses <- c('numeric', 'integer', 'character', 'factor', 'Date')
    if( any(!sapply(data, class) %in% validClasses) )
        stop(paste("The classes of input columns must be one of the following:",paste(validClasses,collapse = ', ')))
    if( !is.null(startDate) && inherits(startDate, "Date"))
        stop("The specified startDate must be of class Date")
    if( !is.null(endDate) && inherits(endDate, "Date"))
        stop("The specified endDate must be of class Date")
    if ( any(!sapply(supports,class) %in% c('numeric', 'integer', 'character', 'factor')))
        stop("All the elements provided in the supports parameter must be of class data.frame")
    
    # Separate analysis data from analysis dates
    dates    <- data[[dateColumnName]]
    data     <- data[, setdiff(colnames(data),dateColumnName), drop = FALSE]
    nColumns <- ncol(data)
    
    if( verbose ){
        message(sprintf("Total number of columns to analyze: %d",nColumns)) 
        message(sprintf("Analysis period: %s",period)) 
    }
    
    dates = lubridate::floor_date(dates, unit = period, week_start = getOption("lubridate.week.start", 7))
    
    # Get variable types, others will not be allowed
    dataClasses  <- sapply(data, class)
    idxNumeric   <- dataClasses == "numeric"
    idxInteger   <- dataClasses == "integer"
    idxCharacter <- dataClasses == "character"
    idxDate      <- dataClasses == "Date"
    idxFactor    <- dataClasses == "factor"
    
    if( verbose ){
        if(any(idxNumeric))message(sprintf("Number of numeric columns: %d",sum(idxNumeric))) 
        if(any(idxInteger))message(sprintf("Number of integer columns: %d",sum(idxInteger))) 
        if(any(idxCharacter))message(sprintf("Number of character columns: %d",sum(idxCharacter))) 
        if(any(idxDate))message(sprintf("Number of Date columns: %d",sum(idxDate)))
        if(any(idxFactor))message(sprintf("Number of factor columns: %d",sum(idxFactor))) 
    }
    
    # Convert Date variables to numbers
    if ( any(idxDate) ){
        data[,idxDate] = lapply(data[,idxDate, drop = FALSE],as.numeric) 
        if( verbose )
            message("Converting Date columns to numeric for distribution analysis")
    }
    
    # Create supports
    supportsNew <- vector("list", nColumns)
    names(supportsNew) <- colnames(data)
    supportsToEstimate <- rep(TRUE,nColumns)
    
    if( !is.null(supports) ) {
        
        suppNamesMatch <- match(names(supports),names(supportsNew))
        
        if ( any(is.na(suppNamesMatch)) )
            warning("The name of one or more elements provided in support do not match with the variable names of the data, these will be ignored.")
        
        if ( any(!is.na(suppNamesMatch)) ){
            
            supportsProvidedIdx = suppNamesMatch[!is.na(suppNamesMatch)]
            supportsToEstimate[supportsProvidedIdx] = FALSE
            
            for (i in 1:length(supportsProvidedIdx)){
                varName <- names(supportsNew)[supportsProvidedIdx[i]]
                supportsNew[[varName]] <- supports[[varName]]
                errorInSupport = switch(dataClasses[varName],
                                        "factor" = {!class(supportsNew[[varName]]) %in% c("factor","character")},
                                        "numeric" = {!is.numeric(supportsNew[[varName]])},
                                        "integer" = {!is.integer(supportsNew[[varName]])},
                                        "character" = {!class(supportsNew[[varName]]) %in% c("factor","character")},
                                        "date" = {!is.integer(supportsNew[[varName]])}
                )
                if ( errorInSupport )
                    stop(sprintf("The provided support for variable %s does not match with its variable type",names(supportsNew)[i]))
            }
            
        }
    }
    
    supports <- supportsNew
    
    if( any(supportsToEstimate) && verbose ){
        message("Estimating supports from data")
        
        allNA = sapply(data[,supportsToEstimate, drop = FALSE], FUN = function(x) all(is.na(x)))
        
        # Exclude from the analysis those variables with no finite values, if any
        if(any(allNA)){
            if( verbose )
                message(sprintf("Removing variables with no finite values: %s", paste(names(data[,supportsToEstimate, drop = FALSE])[allNA], collapse = ", ")))
            warning(sprintf("Removing variables with no finite values: %s", paste(names(data[,supportsToEstimate, drop = FALSE])[allNA], collapse = ", ")))            
            
            data = data[,!allNA]
            nColumns <- ncol(data)
            supports = supports[!allNA]
            supportsToEstimate = supportsToEstimate[!allNA]
            dataClasses = dataClasses[!allNA]
            idxNumeric   <- dataClasses == "numeric"
            idxInteger   <- dataClasses == "integer"
            idxCharacter <- dataClasses == "character"
            idxDate      <- dataClasses == "Date"
            idxFactor    <- dataClasses == "factor"
        }
    }
    
    if ( any(idxFactor & supportsToEstimate) ){
        data[,idxFactor & supportsToEstimate] = lapply(data[,idxFactor & supportsToEstimate], addNA, ifany = TRUE)
        supports[idxFactor & supportsToEstimate] = lapply(data[,idxFactor & supportsToEstimate, drop = FALSE], levels)
    }
    if ( any(idxNumeric & supportsToEstimate) ){
        mins = sapply(data[,idxNumeric & supportsToEstimate, drop = FALSE], min, na.rm = TRUE)
        maxs = sapply(data[,idxNumeric & supportsToEstimate, drop = FALSE], max, na.rm = TRUE)
        supports[idxNumeric & supportsToEstimate] = data.frame(mapply(seq, mins, maxs, length=rep(numericVariablesBins,length(mins))))
        if ( any(mins == maxs) )
            supports[idxNumeric & supportsToEstimate][mins == maxs] = lapply(supports[idxNumeric & supportsToEstimate][mins == maxs], FUN = function(x) x[1])
    }
    if ( any(idxInteger & supportsToEstimate) ){
        mins = sapply(data[,idxInteger & supportsToEstimate, drop = FALSE], min, na.rm = TRUE)
        maxs = sapply(data[,idxInteger & supportsToEstimate, drop = FALSE], max, na.rm = TRUE)
        if(sum(idxInteger & supportsToEstimate)==1){
            supports[idxInteger & supportsToEstimate] = data.frame(mapply(seq, mins, maxs))
        }
        else{
            supports[idxInteger & supportsToEstimate] = mapply(seq, mins, maxs)
        }
    }
    if ( any(idxCharacter & supportsToEstimate) ){
        supports[idxCharacter & supportsToEstimate] = lapply(data[,idxCharacter & supportsToEstimate, drop = FALSE], unique)
    } 
    if ( any(idxDate & supportsToEstimate) ){ # NOTE: treating dates as numerics
        mins = sapply(data[,idxDate & supportsToEstimate, drop = FALSE], min, na.rm = TRUE)
        maxs = sapply(data[,idxDate & supportsToEstimate, drop = FALSE], max, na.rm = TRUE)
        supports[idxDate & supportsToEstimate] = data.frame(mapply(seq, mins, maxs, length=rep(numericVariablesBins,length(mins))))
    }
    
    # Convert factor variables to characters, as used by the xts Objects
    if ( any(idxFactor) )
        data[,idxFactor] = lapply(data[,idxFactor, drop = FALSE],as.character)
    
    # Exclude from the analysis those variables with a single value, if any
    suppLengths = sapply(supports, length)
    idxSuppSingles = suppLengths<2
    if(any(idxSuppSingles)){
        
        if( verbose )
            message(sprintf("Removing variables with less than two distinct values in their supports: %s",paste(colnames(data)[idxSuppSingles],collapse = ", ")))
        warning(paste("The following variable/s have less than two distinct values in their supports and were excluded from the analysis:",paste(colnames(data)[idxSuppSingles],collapse = ", ")))
        
        data = data[,!idxSuppSingles]
        supports = supports[!idxSuppSingles]
        dataClasses = dataClasses[!idxSuppSingles]
        nColumns <- ncol(data)
    }
    
    if(nColumns == 0)
        stop("Zero remaining variables to be analyzed.")
    
    # Estimate the Data Temporal Map
    dataClassesPost = sapply(data, class)
    results <- vector("list", nColumns)
    
    if( verbose )
        message("Estimating the data temporal maps")
    
    for(i in 1:nColumns){
        
        if( verbose )
            message(sprintf("Estimating the DataTemporalMap of variable '%s'",colnames(data)[i]))
        
        dataxts = xts::xts(data[,i],order.by = dates)
        
        if (!is.null(startDate) || !is.null(endDate)){
            
            if (is.null(startDate))
                startDate = min(dates)
            if (is.null(endDate))
                endDate = max(dates)
            
            dataxts = dataxts[paste(startDate,endDate, sep="/")]
        }
        
        map = switch(period,
                     "week" = xts::apply.weekly(dataxts, FUN = estimateAbsoluteFrequencies, varclass = dataClassesPost[i], support = supports[[i]], numericSmoothing = numericSmoothing),
                     "month" = xts::apply.monthly(dataxts, FUN = estimateAbsoluteFrequencies, varclass = dataClassesPost[i], support = supports[[i]], numericSmoothing = numericSmoothing),
                     "year" = xts::apply.yearly(dataxts, FUN = estimateAbsoluteFrequencies, varclass = dataClassesPost[i], support = supports[[i]], numericSmoothing = numericSmoothing)
        )
        
        datesMap <- as.Date(zoo::index(map))
        seqDateFull <- seq.Date(min(datesMap),max(datesMap), by = period)
        dateGapsSmoothingDone = FALSE
        
        if (length(datesMap) != length(seqDateFull)){
            
            nGaps = length(seqDateFull)-length(datesMap)
            
            seqDateZoo = zoo::zoo(NULL,seqDateFull)
            map = xts::merge.xts(map,seqDateZoo,fill = NA)
            
            if(dateGapsSmoothing){
                map = zoo::na.approx(map)
                if( verbose )
                    message(sprintf("-'%s': %d %s date gaps filed by linear smoothing",colnames(data)[i],nGaps,period))
                dateGapsSmoothingDone = TRUE
            } else{
                if( verbose )
                    message(sprintf("-'%s': %d %s date gaps filed by NAs",colnames(data)[i],nGaps,period))
            }
            
            datesMap <- as.Date(zoo::index(map))
        } else {
            if(verbose && dateGapsSmoothing)
                message(sprintf("-'%s': no date gaps, date gap smoothing was not applied",colnames(data)[i]))
        }
        
        countsMap = zoo::coredata(map)
        probabilityMap = sweep(countsMap,1,rowSums(countsMap),"/")
        
        if(dataClasses[i] == "Date"){
            support = data.frame(as.Date(supports[[i]],origin = lubridate::origin))
        } else if(dataClasses[i] %in% c("character","factor")){
            support = data.frame(as.character(supports[[i]]), stringsAsFactors=FALSE)
        } else {
            support = data.frame(supports[[i]])
        }
        
        if(dateGapsSmoothingDone && any(is.na(probabilityMap)))
            warning(sprintf("Date gaps smoothing was performed in '%s' variable but some gaps will still be reflected in the resultant probabilityMap (this is generally due to temporal heatmap sparsity)",colnames(data)[i]))
        
        probMap = new( "DataTemporalMap",
                       probabilityMap = probabilityMap,
                       countsMap      = countsMap, 
                       dates          = datesMap, 
                       support        = support, 
                       variableName   = colnames(data)[i], 
                       variableType   = dataClasses[i], 
                       period         = period)
        results[[i]] = probMap
    }
    
    if (nColumns > 1){
        names(results) <- colnames(data)
        if( verbose )
            message("Returning results as a named list of DataTemporalMap objects")
        return(results)
    }
    else {
        if( verbose )
            message("Returning results as an individual DataTemporalMap object")
        return(results[[1]])
    }
    
    
}

estimateAbsoluteFrequencies <- function(data = NULL, varclass = NULL, support = NULL, numericSmoothing = FALSE){
    data = zoo::coredata(data);
    switch(varclass,
           "character" = {
               dataTable = as.data.frame(table(factor(data,levels = support, exclude = NULL)))
               map = dataTable$Freq
           },
           "numeric" = {
               if (all(is.na(data))){
                   map = as.numeric(rep(NA,length(support)))
               } else {
                   if(!numericSmoothing){
                       histSupport = c(support,support[length(support)]+(support[length(support)]-support[length(support)-1]))
                       data = data[data>=min(histSupport) & data<max(histSupport)]
                       map = graphics::hist(data,histSupport,plot=FALSE,right=FALSE,include.lowest=TRUE)$counts
                   }
                   else{
                       if (sum(!is.na(data))<4){
                           warning(paste0("Estimating a 1-dimensional kernel density smoothing with less than 4 data points can result in an inaccurate estimation.",
                                          " For more information see 'Density Estimation for Statistics and Data Analysis, Bernard. W. Silverman, CRC ,1986', chapter 4.5.2 'Required sample size for given accuracy'."))
                       }
                       if (sum(!is.na(data))<2){
                           data = rep(data[!is.na(data)],2)
                           ndata = 1
                       } else
                       {
                           ndata = sum(!is.na(data))
                       }
                       dataDensity = stats::density(data, na.rm = T, from = support[1], to = support[length(support)], n = length(support))
                       map = (dataDensity$y/sum(dataDensity$y))*ndata
                   }
               }
           },
           "integer" = {
               if (all(is.na(data))){
                   map = as.integer(rep(NA,length(support)))
               } else {
                   histSupport = c(support,support[length(support)]+(support[length(support)]-support[length(support)-1]))
                   data = data[data>=min(histSupport) & data<max(histSupport)]
                   map = graphics::hist(data,histSupport,plot=FALSE,right=FALSE,include.lowest=TRUE)$counts
               }
           },
           stop(sprintf('data class %s not valid for distribution estimation.',varclass))
    )
    map
}

# Copyright 2019 Biomedical Data Science Lab, Universitat Politècnica de València (Spain) - Department of Biomedical Informatics, Harvard Medical School (US)
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
# http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' Estimates an Information Geometric Temporal plot projection
#'
#' Estimates an \code{IGTProjection} object from a \code{DataTemporalMap} object.
#'
#' @name estimateIGTProjection
# @docType methods
#' @rdname estimateIGTProjection-methods
#' @aliases estimateIGTProjection,IGTProjection-method
#' @param dataTemporalMap of class \code{DataTemporalMap} object.
#' @param dimensions \code{numeric} integer value indicating the number of dimensions 
#' for the projection.
#' @param startDate a Date object indicating the date at which to start the analysis, 
#' in case of being different from the first chronological date in the date column 
#' (the default).
#' @param endDate a Date object indicating the date at which to end the analysis, 
#' in case of being different from the last chronological date in the date column 
#' (the default).
#' @param embeddingType the type of embedding to apply to the dissimilarity matrix of time batches
#' in order to obtain the non-parametric Statistical Manifold, from "classicalmds" and "nonmetricmds", 
#' with "classicalmds" as default. "classicalmds" uses the base R stats::cmdscale function, while "nonmetricmds"
#' uses the MASS:isoMDS function. The returned stress format will depend on the selected embedding type:
#' "classicalmds" returns 1-GOF as returned by stats::cmdscale function, "nonmetricmds" returns the final stress
#' in percent, as returned by the MASS::isoMDS function
#' @return An \code{IGTProjection} object containing the projected coordinates of each
#' temporal batch in the embedded non-parametric Statistical Manifold, as well as the
#' embedding stress according to the embeddingType.
#' @examples
#' load(system.file("extdata",
#'                  "variabilityDemoNHDSdiagcode1-phewascode.RData",
#'                   package="EHRtemporalVariability"))
#' igtProj <- estimateIGTProjection( dataTemporalMap = probMaps$`diagcode1-phewascode`, 
#' dimensions      = 3, 
#' startDate       = "2000-01-01", 
#' endDate         = "2010-12-31")
#' 
#' \dontrun{
#' 
#' # For additional and larger examples download the following .Rdata file:
#' 
#' gitHubUrl  <- 'http://github.com/'
#' gitHubPath <- 'hms-dbmi/EHRtemporalVariability-DataExamples/'
#' gitHubFile <- 'raw/master/variabilityDemoNHDS.RData'
#' inputFile  <-  paste0(gitHubUrl, gitHubPath, gitHubFile)
#' 
#' load(url(inputFile))
#' igtProj <- estimateIGTProjection( dataTemporalMap = probMaps[[1]], 
#' dimensions      = 3, 
#' startDate       = "2000-01-01", 
#' endDate         = "2010-12-31")
#' }
#' @exportMethod estimateIGTProjection
setGeneric (name       = "estimateIGTProjection",
            valueClass = "IGTProjection",
            def        = function(dataTemporalMap, dimensions = 3, startDate = NULL, endDate = NULL, embeddingType = "classicalmds")
            {
                standardGeneric("estimateIGTProjection")
            }
)


#' Data Temporal heatmap
#'
#' Plots a Data Temporal heatmap from an \code{DataTemporalMap} object.
#'
#' @name plotDataTemporalMap
#' @rdname plotDataTemporalMap-methods
#' @param dataTemporalMap of class \code{DataTemporalMap}
#' @param absolute indicates if the heatmap frequency values are absolute or relative. 
#' By default \code{FALSE}.
#' @param startValue indicates the first value to display in the heatmap.
#' By default 1.
#' @param endValue indicates the last value to display in the heatmap.
#' By default the last value of the \code{DataTemporalMap} object.
#' @param startDate a Date object indicating the first date to be displayed in the heatmap. 
#' By default the first date of the \code{DataTemporalMap} object.
#' @param endDate a Date object indicating the last date to be displayed in the heatmap. 
#' By default the last date of the \code{DataTemporalMap} object.
#' @param sortingMethod the method to sort data in the Y axis of the heatmap from "frequency" and 
#' "alphabetical", with "frequency" as default.
#' @param colorPalette color palette to be used. The default "Spectral" palette shows a 
#' color temperature scheme from blue, through yellow, to red (see "Spectral" palette in
#'  RColorBrewer package). The four remaining options are better suited for those with 
#'  colorblindness, including "Viridis", "Magma", and their reversed versions 
#'  "Viridis-reversed" and "Magma-reversed" (see "Viridis" and "Magma" palettes in the
#'  Viridis package).
#' @param mode indicates the plot mode as a 'heatmap' (default) or 'series'.
#'  The other config parameters for the heatmap plot also apply for the series plot.
#' @return A plot object based on the \code{plotly} package.
#' @examples
#' load(system.file("extdata",
#'                  "variabilityDemoNHDSdiagcode1-phewascode.RData",
#'                   package="EHRtemporalVariability"))
#' 
#' p <- plotDataTemporalMap(dataTemporalMap =  probMaps[[1]],
#'                     colorPalette    = "Spectral",
#'                     startValue = 2,
#'                     endValue = 40)
#' p
#' 
#' p <- plotDataTemporalMap(dataTemporalMap =  probMaps[[1]],
#'                     colorPalette    = "Spectral",
#'                     startValue = 2,
#'                     endValue = 40,
#'                     mode = "series")
#' p
#' 
#' \dontrun{
#' 
#' # For additional and larger examples download the following .Rdata file:
#' 
#' gitHubUrl  <- 'http://github.com/'
#' gitHubPath <- 'hms-dbmi/EHRtemporalVariability-DataExamples/'
#' gitHubFile <- 'raw/master/variabilityDemoNHDS.RData'
#' inputFile  <-  paste0(gitHubUrl, gitHubPath, gitHubFile)
#' 
#' load(url(inputFile))
#' plotDataTemporalMap(probMaps$`diagcode1-phewascode`, startValue = 2, endValue = 40)
#' }
#' @exportMethod plotDataTemporalMap
#' @importFrom stats complete.cases setNames
setGeneric (name       = "plotDataTemporalMap",
            valueClass = c("plotly","htmlwidget"),
            def        = function(dataTemporalMap, absolute = FALSE, startValue = 1, endValue = ncol(dataTemporalMap@probabilityMap), startDate = min(dataTemporalMap@dates), endDate = max(dataTemporalMap@dates), sortingMethod = 'frequency', colorPalette = 'Spectral', mode = 'heatmap')
            {
                standardGeneric("plotDataTemporalMap")
            }
)

#' Information Geometric Temporal plot
#'
#' Plots an interactive Information Geometric Temporal (IGT) plot from an \code{IGTProjection} object.
#' An IGT plot visualizes the variability among time batches in a data repository in a 2D or 3D plot.
#' Time batches are positioned as points where the distance between them represents the probabilistic 
#' distance between their distributions (currently Jensen-Shannon distance, more distances will be 
#' supported in the future).
#' To track the temporal evolution, temporal batches are labeled to show their date and 
#' colored according to their season or period, according to the analysis period, as follows.
#' If period=="year" the label is "yy" (2 digit year) and the color is according to year.
#' If period=="month" the label is "yym" (yy + abbreviated month*) and the color is according 
#' to the season (yearly).
#' If period=="week" the label is "yymmw" (yym + ISO week number in 1-2 digit) and the color is 
#' according to the season (yearly). An estimated smoothed trajectory of the information evolution
#' over time can be shown using the optional "trajectory" parameter.
#' *Month abbreviations: \{'J', 'F', 'M', 'A', 'm', 'j', 'x', 'a', 'S', 'O', 'N', 'D'\}.
#'
#' Note that since the projection is based on multidimensional scaling, a 2 dimensional 
#' projection entails a loss of information compared to a 3 dimensional projection. E.g., periodic 
#' variability components such as seasonal effect can be hindered by an abrupt change or a general trend.
#'
#' @name plotIGTProjection
#' @rdname plotIGTProjection-methods
#' @param igtProjection of class \code{IGTProjection}
#' @param dimensions number of dimensions of the plot, 2 or 3 (3 by default)
#' @param startDate a Date object indicating the first date to be displayed in the IGT plot. 
#' By default the first date of the \code{IGTProjection} object.
#' @param endDate a Date object indicating the last date to be displayed in the IGT plot 
#' By default the last date of the \code{IGTProjection} object.
#' @param colorPalette color palette to be used. The default "Spectral" palette shows a color temperature 
#' scheme from blue, through yellow, to red (see "Spectral" palette in RColorBrewer package). 
#' The four remaining options are better suited for those with colorblindness, including "Viridis", 
#' "Magma", and their reversed versions "Viridis-reversed" and "Magma-reversed" (see "Viridis" and 
#' "Magma" palettes in the Viridis package).
#' @param trajectory whether to show an estimated trajectory of the information evolution over time. 
#' By default \code{FALSE}.
#' @return A plot object based on the \code{plotly} package.
#' @examples
#' load(system.file("extdata",
#'                  "variabilityDemoNHDSdiagcode1-phewascode.RData",
#'                   package="EHRtemporalVariability"))
#' 
#' p <- plotIGTProjection( igtProjection   =  igtProjs[[1]],
#'                         colorPalette    = "Spectral",
#'                         dimensions      = 2)
#' p
#' 
#' \dontrun{
#' 
#' # For additional and larger examples download the following .Rdata file:
#' 
#' gitHubUrl  <- 'http://github.com/'
#' gitHubPath <- 'hms-dbmi/EHRtemporalVariability-DataExamples/'
#' gitHubFile <- 'raw/master/variabilityDemoNHDS.RData'
#' inputFile  <-  paste0(gitHubUrl, gitHubPath, gitHubFile)
#' 
#' load(url(inputFile)) 
#' plotIGTProjection(igtProjs$`diagcode1-phewascode`, dimensions = 3)
#' } 
#' @exportMethod plotIGTProjection
#' @importFrom methods .valueClassTest new
setGeneric (name       = "plotIGTProjection",
            valueClass = c("plotly","htmlwidget"),
            def        = function(igtProjection, dimensions = 3, startDate = min(igtProjection@dataTemporalMap@dates), endDate = max(igtProjection@dataTemporalMap@dates), colorPalette = "Spectral", trajectory = FALSE)
            {
                standardGeneric("plotIGTProjection")
            }
)

#' Trims a \code{DataTemporalMap}
#'
#' Trims a \code{DataTemporalMap} object between an start and end date. If one is not specified it takes 
#' as default the first/last chronological date in the input \code{DataTemporalMap}.
#'
#' @name trimDataTemporalMap
#' @rdname trimDataTemporalMap-methods
#' @param dataTemporalMap of class \code{DataTemporalMap}.
#' @param startDate \code{Date} indicating the start date to trim from.
#' @param endDate \code{Date} indicating the end date to trim to.
#' @return A \code{DataTemporalMap} object between the specified dates.
#' @examples
#' load(system.file("extdata",
#'                  "variabilityDemoNHDSdiagcode1-phewascode.RData",
#'                   package="EHRtemporalVariability"))
#' 
#' probMapTrimmed <- trimDataTemporalMap( 
#'                          dataTemporalMap = probMaps[[1]],
#'                          startDate       = "2005-01-01",
#'                          endDate         = "2008-12-01"
#' )
#' @exportMethod trimDataTemporalMap
setGeneric (name = "trimDataTemporalMap",
            valueClass = "DataTemporalMap",
            def        = function(dataTemporalMap, startDate = min(dataTemporalMap@dates), endDate = max(dataTemporalMap@dates))
            {
                standardGeneric("trimDataTemporalMap")
            }
)

# Copyright 2019 Biomedical Data Science Lab, Universitat Politècnica de València (Spain) - Department of Biomedical Informatics, Harvard Medical School (US)
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
# http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

igtProjectionCore <- function(dataTemporalMap = NULL, dimensions = 3, embeddingType = "classicalmds") {
    
    dates = dataTemporalMap@dates
    temporalMap = dataTemporalMap@probabilityMap
    ndates <- length(dates)
    
    
    ## Loop version
    
    dissimMatrix = matrix(data=0,nrow=ndates,ndates)
    for(i in 1:(ndates-1)){
        for(j in (i+1):ndates){
            dissimMatrix[i,j] = sqrt(jsdiv(temporalMap[i,],temporalMap[j,]))
            dissimMatrix[j,i] = dissimMatrix[i,j]
        }
    }
    
    ## Vectorized version
    ## NOTE: The following vectorized code is ~20% faster than the above but vec1,vec2 allocation can fail if not enough memory
    #
    # idx = which(lower.tri(matrix(, nrow = ndates, ncol = ndates), diag = FALSE), arr.ind=T)
    # 
    # vec1 = temporalMap[idx[,2],]
    # vec2 = temporalMap[idx[,1],]
    # 
    # mvec = 0.5 * (vec1 + vec2)
    # jsdists = sqrt(0.5 * (rowSums(vec1 * log2(vec1/mvec), na.rm = TRUE) + rowSums(vec2 * log2(vec2/mvec), na.rm = TRUE)))
    # 
    # dissimMatrix = matrix(data=0, nrow = ndates, ncol = ndates)
    # dissimMatrix[lower.tri(dissimMatrix, diag = FALSE)] = jsdists
    # dissimMatrix = as.dist(dissimMatrix)
    
    embeddingResults = switch(embeddingType,
                            "classicalmds" = {stats::cmdscale(dissimMatrix, eig = FALSE, k = dimensions, list. = TRUE)},
                            "nonmetricmds" = {MASS::isoMDS(dissimMatrix, trace = FALSE, k = dimensions)}
    )
    
    igtProj = switch(embeddingType,
                     "classicalmds" = {IGTProjection(dataTemporalMap = dataTemporalMap, projection = embeddingResults$points, embeddingType = embeddingType, stress = 1-embeddingResults$GOF)},
                     "nonmetricmds" = {IGTProjection(dataTemporalMap = dataTemporalMap, projection = embeddingResults$points, embeddingType = embeddingType, stress = embeddingResults$stress)}
    )

    return(igtProj)
}

# Copyright 2019 Biomedical Data Science Lab, Universitat Politècnica de València (Spain) - Department of Biomedical Informatics, Harvard Medical School (US)
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
# http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' @rdname estimateIGTProjection-methods
#' @aliases estimateIGTProjection,IGTProjection-method
setMethod(f="estimateIGTProjection",
          signature  = c("DataTemporalMap"),
          definition = function(dataTemporalMap, dimensions, startDate, endDate, embeddingType)
          {
              if(is.null(dataTemporalMap))
                  stop("dataTemporalMap of class DataTemporalMap must be provided")
              
              if(dimensions < 2 || dimensions > length(dataTemporalMap@dates))
                  stop("dimensions must be between 2 and length(dataTemporalMap@dates)")
              
              if (!is.null(startDate) && !is.null(endDate)){
                  
                  dataTemporalMap = trimDataTemporalMap(dataTemporalMap, startDate = startDate, endDate = endDate)
                  
              } else {
                  
                  if (!is.null(startDate))
                      dataTemporalMap = trimDataTemporalMap(dataTemporalMap, startDate = startDate)
                  
                  if (!is.null(endDate))
                      dataTemporalMap = trimDataTemporalMap(dataTemporalMap, endDate = endDate)
              }
              
              if (!embeddingType %in% c("classicalmds", "nonmetricmds"))
                  stop("embeddingType must be one of classicalmds or nonmetricmds")
              
              value <- igtProjectionCore(dataTemporalMap = dataTemporalMap, dimensions = dimensions, embeddingType = embeddingType)
              return(value)
          }
)

#' @rdname plotDataTemporalMap-methods
setMethod(f = "plotDataTemporalMap",
          signature  = "DataTemporalMap",
          definition = function(dataTemporalMap, absolute, startValue, endValue, startDate, endDate, sortingMethod, colorPalette, mode ){
              
              if (!mode %in% c("heatmap", "series"))
                  stop("mode must be one of heatmap or series")
              
              if (!colorPalette %in% c("Spectral", "Viridis", "Magma", "Viridis-reversed", "Magma-reversed"))
                  stop("colorPalette must be one of Spectral, Viridis, Magma, Viridis-reversed or Magma-reversed")
              
              if (!is.logical(absolute))
                  stop("absolute must be a logical value")
              
              if (startValue < 1)
                  stop("startValue must be greater or equal than 1")
              
              if (!sortingMethod %in% c("frequency", "alphabetical"))
                  stop("sortMethod must be one of frequency or alphabetical")
              
              vals = seq(0, 1, length.out = 100)
              cols = switch(colorPalette,
                            "Spectral"         = scales::col_numeric("Spectral", domain = NULL)(vals),
                            "Viridis"          = scales::col_numeric(viridis::viridis(100), domain = NULL)(vals),
                            "Magma"            = scales::col_numeric(viridis::magma(100), domain = NULL)(vals),
                            "Viridis-reversed" = scales::col_numeric(viridis::viridis(100,direction = -1), domain = NULL)(vals),
                            "Magma-reversed"   = scales::col_numeric(viridis::magma(100,direction = -1), domain = NULL)(vals)
              )
              
              colorScale = setNames(data.frame(vals, cols),NULL)
              
              temporalMap = switch(absolute+1,
                                   xts::xts(dataTemporalMap@probabilityMap, order.by = dataTemporalMap@dates),
                                   xts::xts(dataTemporalMap@countsMap, order.by = dataTemporalMap@dates)
              )
              temporalMap = temporalMap[paste(startDate,endDate, sep="/")]
              dates = zoo::index(temporalMap)
              temporalMap = zoo::coredata(temporalMap)
              
              support = dataTemporalMap@support
              variableType = dataTemporalMap@variableType
              
              if (variableType %in% c('factor','character')) {
                  if (sortingMethod  %in% 'frequency'){
                      supportOrder = order(colSums(temporalMap, na.rm = TRUE),decreasing = TRUE)
                  } else {
                      supportOrder = order(support[,1],decreasing = FALSE)
                  }
                  
                  support = support[supportOrder,, drop = FALSE]
                  temporalMap = temporalMap[,supportOrder]
                  
                  anySuppNa = is.na(support)
                  if(any(anySuppNa))
                      support[anySuppNa] = "<NA>"
              }
              
              # if (variableType %in% 'factor') {
              #     support = as.character(support)
              # }
              
              if ( endValue > ncol(temporalMap) ){
                  endValue = ncol(temporalMap)
              }
              
              f <- list(
                  #family = "Courier New, monospace",
                  size = 18,
                  color = "#7f7f7f"
              )
              x <- list(
                  title = "Date",
                  titlefont = f,
                  type = "date"
              )
              y <- list(
                  title = dataTemporalMap@variableName,
                  titlefont = f,
                  tickfont = 14,
                  automargin = TRUE,
                  type = switch(variableType, "character" = "category", "factor" = "category", "numeric" = "-" )
              )
              m <- list(
                  l = min(max(nchar(support[startValue:endValue,1]))*(50),125)
              )
              
              if (mode == 'heatmap') {
                  p <- plotly::plot_ly(x=dates, y=support[startValue:endValue,1], z = t(as.data.frame(temporalMap[,startValue:endValue])),
                                       type = "heatmap", colorscale = colorScale, reversescale = TRUE) %>%
                      plotly::config(staticPlot = FALSE, displayModeBar = TRUE, editable = FALSE,
                                     sendData = FALSE, displaylogo = FALSE, 
                                     modeBarButtonsToRemove = list("sendDataToCloud","hoverCompareCartesian"))%>%
                      plotly::layout(xaxis = x, yaxis = y, title = ifelse(absolute, "Absolute frequencies data temporal heatmap", "Probability distribution data temporal heatmap" )) %>%
                      plotly::layout(margin = m)
              }
              else if (mode == 'series') {
                  seriesNum=startValue:endValue
                  title = switch(absolute+1, "Relative frequency", "Absolute frequency" )
                  if (colorPalette == "Spectral"){
                      p <- plotly::plot_ly(x=dates, y=temporalMap[,seriesNum[1]], name = support[seriesNum[1],1], type = 'scatter', mode = 'lines'
                      ) %>%
                          plotly::layout(title = paste0("Evolution of ",dataTemporalMap@variableName),
                                         xaxis = list(title = "Date"),
                                         yaxis = list (title = title)) %>%
                          plotly::config(staticPlot = FALSE, displayModeBar = TRUE, editable = FALSE,
                                         sendData = FALSE, displaylogo = FALSE, 
                                         modeBarButtonsToRemove = list("sendDataToCloud","hoverCompareCartesian"))
                      for (i in 2:length(seriesNum)) {
                          p = p %>% plotly::add_trace(y=temporalMap[,seriesNum[i]], name = support[seriesNum[i],1], mode = 'lines'
                          )
                      }
                  }
                  else{
                      maxColors = 6
                      vals = seq(0, 1, length.out = maxColors)
                      cols = switch(colorPalette,
                                    "Viridis"          = scales::col_numeric(viridis::viridis(100), domain = NULL)(vals),
                                    "Magma"            = scales::col_numeric(viridis::magma(100), domain = NULL)(vals),
                                    "Viridis-reversed" = scales::col_numeric(viridis::viridis(100,direction = -1), domain = NULL)(vals),
                                    "Magma-reversed"   = scales::col_numeric(viridis::magma(100,direction = -1), domain = NULL)(vals)
                      )
                      colorScale = setNames(data.frame(vals, cols),NULL)
                      p <- plotly::plot_ly(x=dates, y=temporalMap[,seriesNum[1]], name = support[seriesNum[1],1], type = 'scatter', mode = 'lines',
                                           line = list(color = colorScale[[1,2]])) %>%
                          plotly::layout(title = paste0("Evolution of ",dataTemporalMap@variableName),
                                         xaxis = list(title = "Date"),
                                         yaxis = list (title = title)) %>%
                          plotly::config(staticPlot = FALSE, displayModeBar = TRUE, editable = FALSE,
                                         sendData = FALSE, displaylogo = FALSE, 
                                         modeBarButtonsToRemove = list("sendDataToCloud","hoverCompareCartesian"))
                      for (i in 2:length(seriesNum)) {
                          p = p %>% plotly::add_trace(y=temporalMap[,seriesNum[i]], name = support[seriesNum[i],1], mode = 'lines',
                                                      line = list(color = colorScale[[(i-1) %% maxColors + 1,2]]))
                      }
                  }
              }
              
              return(p)
          }
)

#' @rdname plotIGTProjection-methods
setMethod(f="plotIGTProjection",
          signature  = "IGTProjection",
          definition = function(igtProjection, dimensions, startDate, endDate, colorPalette, trajectory){
              
              if (dimensions < 2 || dimensions > 3)
                  stop("currently IGT plot can only be made on 2 or 3 dimensions, please set dimensions parameter accordingly")
              
              if (!colorPalette %in% c("Spectral", "Viridis", "Magma", "Viridis-reversed", "Magma-reversed"))
                  stop("colorPalette must be one of Spectral, Viridis, Magma, Viridis-reversed or Magma-reversed")
              
              
              dateidxs = igtProjection@dataTemporalMap@dates >= startDate & igtProjection@dataTemporalMap@dates <= endDate
              
              dates = igtProjection@dataTemporalMap@dates[dateidxs]
              projection = igtProjection@projection[dateidxs,]
              ndates = length(dates)
              
              if( trajectory ){
                  igtTrajectory = estimateIGTTrajectory(igtProjection)
                  trajectorydatesidxs = igtTrajectory$dates >= startDate & igtTrajectory$dates <= endDate
                  igtTrajectoryPoints = igtTrajectory$points[trajectorydatesidxs,]
              }
              
              if (igtProjection@dataTemporalMap@period == "year"){
                  yearcolor = switch(colorPalette,
                                     "Spectral"         = grDevices::colorRampPalette(RColorBrewer::brewer.pal(11,"Spectral"))(ndates),
                                     "Viridis"          = viridis::viridis(ndates),
                                     "Magma"            = viridis::magma(ndates),
                                     "Viridis-reversed" = viridis::viridis(ndates, direction = -1),
                                     "Magma-reversed"   = viridis::magma(ndates, direction = -1)
                  )
              }
              else { # month and week
                  
                  vals = seq(0, 1, length.out = 100)
                  colorlist = switch(colorPalette,
                                     "Spectral"         = grDevices::colorRampPalette(RColorBrewer::brewer.pal(11,"Spectral"))(128),
                                     "Viridis"          = viridis::viridis(128),
                                     "Magma"            = viridis::magma(128),
                                     "Viridis-reversed" = viridis::viridis(128, direction = -1),
                                     "Magma-reversed"   = viridis::magma(128, direction = -1)
                  )
                  
                  dperiod = switch(igtProjection@dataTemporalMap@period, "month" = 12, "week" = 53)
                  
                  colorlist = rev(colorlist)
                  colorlist = c(colorlist, rev(colorlist))
                  periodcolor = colorlist[round(seq(1,256,length=(dperiod+1)))]
                  periodcolor = periodcolor[1:dperiod]
                  months = c('J', 'F', 'M', 'A', 'm', 'j', 'x', 'a', 'S', 'O', 'N', 'D')
                  monthsLong = c('Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec')
              }
              
              if (dimensions == 2) {
                  
                  p <- plotly::plot_ly() %>%
                      plotly::config(staticPlot = FALSE, displayModeBar = TRUE, editable = FALSE,
                                     sendData = FALSE, displaylogo = FALSE,
                                     modeBarButtonsToRemove = list("sendDataToCloud","hoverCompareCartesian")) %>%
                      plotly::layout(showlegend = FALSE,
                                     xaxis = list(title = 'D1', scaleanchor = "x"),
                                     yaxis = list(title = 'D2', scaleanchor = "x"),
                                     title = "Information Geometric Temporal (IGT) plot"
                      )
                  
                  if (igtProjection@dataTemporalMap@period == "year"){
                      for(i in 1:ndates){
                          p <- plotly::add_text(p, x = projection[i,1], y = projection[i,2],
                                                hoverinfo = 'text',
                                                # hovertext = paste('Year: ',format(dates[i],"%Y")),
                                                hovertext = paste(format(dates[i],"%Y")),
                                                text = paste(format(dates[i],"%Y")),
                                                textfont = list(size = 14, color = yearcolor[i]), textposition = "middle center")
                      }
                  }
                  else if (igtProjection@dataTemporalMap@period == "month"){
                      cidx = as.numeric(format(dates,'%m'))
                      for(i in 1:ndates){
                          p <- plotly::add_text(p, x = projection[i,1], y = projection[i,2],
                                                hoverinfo = 'text',
                                                # hovertext = paste('Year: ',format(dates[i],"%Y"),'\nMonth: ',monthsLong[cidx[i]]),
                                                hovertext = paste(format(dates[i],"%Y"),'-',monthsLong[cidx[i]]),
                                                text = paste(format(dates[i],"%y"),months[cidx[i]],sep=''),
                                                textfont = list(size = 14, color = periodcolor[cidx[i]]), textposition = "middle center")
                      }
                  }
                  else if (igtProjection@dataTemporalMap@period == "week"){
                      cidxw = as.numeric(lubridate::isoweek(dates))
                      cidxm = as.numeric(format(dates,'%m'))
                      for(i in 1:ndates){
                          p <- plotly::add_text(p, x = projection[i,1], y = projection[i,2],
                                                hoverinfo = 'text',
                                                # hovertext = paste('Year: ',format(dates[i],"%Y"),'\nMonth: ',monthsLong[cidx[i]],'\nWeek: ',cidxw[i]),
                                                hovertext = paste(format(dates[i],"%Y"),'-',monthsLong[cidxm[i]],'-w',cidxw[i]),
                                                text = paste(format(dates[i],"%y"),months[cidxm[i]],cidxw[i],sep=''),
                                                textfont = list(size = 14, color = periodcolor[cidxw[i]]), textposition = "middle center")
                      }
                  }
                  
                  if( trajectory ){
                      p <- plotly::add_trace(p, x = igtTrajectoryPoints$D1, y = igtTrajectoryPoints$D2,
                                             type = 'scatter', mode = 'lines', line = list(color = "#21908C", width = 1),
                                             hovertext = sprintf("Approx. date: %s",igtTrajectory$dates[trajectorydatesidxs])) %>% plotly::hide_colorbar()
                  }
                  
                  return(p)
                  
              } else if (dimensions == 3) {
                  
                  p <- plotly::plot_ly() %>%
                      plotly::config(staticPlot = FALSE, displayModeBar = TRUE, editable = FALSE,
                                     sendData = FALSE, displaylogo = FALSE,
                                     modeBarButtonsToRemove = list("sendDataToCloud","hoverCompareCartesian")) %>%
                      plotly::layout(showlegend = FALSE, scene = list(xaxis = list(title = 'D1', scaleanchor = "x"),
                                                                      yaxis = list(title = 'D2', scaleanchor = "x"),
                                                                      zaxis = list(title = 'D3', scaleanchor = "x")),
                                     title = "Information Geometric Temporal (IGT) plot"
                      )
                  
                  if (igtProjection@dataTemporalMap@period == "year"){
                      for(i in 1:ndates){
                          p <- plotly::add_text(p, x = projection[i,1], y = projection[i,2], z = projection[i,3],
                                                hoverinfo = 'text',
                                                # hovertext = paste('Year: ',format(dates[i],"%Y")),
                                                hovertext = paste(format(dates[i],"%Y")),
                                                text = paste(format(dates[i],"%Y")),
                                                textfont = list(size = 14, color = yearcolor[i]), textposition = "middle center")
                      }
                  }
                  else if (igtProjection@dataTemporalMap@period == "month"){
                      cidx = as.numeric(format(dates,'%m'))
                      for(i in 1:ndates){
                          p <- plotly::add_text(p, x = projection[i,1], y = projection[i,2], z = projection[i,3],
                                                hoverinfo = 'text',
                                                # hovertext = paste('Year: ',format(dates[i],"%Y"),'\nMonth: ',monthsLong[cidx[i]]),
                                                hovertext = paste(format(dates[i],"%Y"),'-',monthsLong[cidx[i]]),
                                                text = paste(format(dates[i],"%y"),months[cidx[i]],sep=''),
                                                textfont = list(size = 14, color = periodcolor[cidx[i]]), textposition = "middle center")
                      }
                      # textfonts = lapply(monthcolor[cidx], function(x) list(size = 14, color = x))
                      # for(i in 1:ndates){
                      #     p <- add_text(p, x = projection[i,1], y = projection[i,2], z = projection[i,3],
                      #                   text = paste(format(dates[i],"%y"),months[cidx[i]],sep=''),
                      #                   textfont = textfonts[[i]], textposition = "middle center")
                      # }
                  }
                  else if (igtProjection@dataTemporalMap@period == "week"){
                      cidxw = as.numeric(lubridate::isoweek(dates))
                      cidxm = as.numeric(format(dates,'%m'))
                      for(i in 1:ndates){
                          p <- plotly::add_text(p, x = projection[i,1], y = projection[i,2], z = projection[i,3],
                                                hoverinfo = 'text',
                                                # hovertext = paste('Year: ',format(dates[i],"%Y"),'\nMonth: ',monthsLong[cidx[i]],'\nWeek: ',cidxw[i]),
                                                hovertext = paste(format(dates[i],"%Y"),'-',monthsLong[cidxm[i]],'-w',cidxw[i]),
                                                text = paste(format(dates[i],"%y"),months[cidxm[i]],cidxw[i],sep=''),
                                                textfont = list(size = 14, color = periodcolor[cidxw[i]]), textposition = "middle center")
                      }
                  }
                  
                  if( trajectory ){
                      p <- plotly::add_paths(p, x = igtTrajectoryPoints$D1, y = igtTrajectoryPoints$D2, z = igtTrajectoryPoints$D3,
                                             color = 1:nrow(igtTrajectoryPoints), hovertext = sprintf("Approx. date: %s",igtTrajectory$dates[trajectorydatesidxs])) %>% plotly::hide_colorbar()
                  }
                  
                  return(p)
                  
              }
              
          }
)


#' @rdname trimDataTemporalMap-methods
setMethod(f="trimDataTemporalMap",
          signature  = "DataTemporalMap",
          definition = function(dataTemporalMap, startDate = min(dataTemporalMap@dates), endDate = max(dataTemporalMap@dates))
          {
              temporalMap <- xts::xts(dataTemporalMap@probabilityMap, order.by = dataTemporalMap@dates)
              temporalMap <- temporalMap[paste(startDate,endDate, sep="/")]
              dates       <- zoo::index(temporalMap)
              temporalMap <- zoo::coredata(temporalMap)
              
              temporalCountsMap <- xts::xts(dataTemporalMap@countsMap, order.by = dataTemporalMap@dates)
              temporalCountsMap <- temporalCountsMap[paste(startDate,endDate, sep="/")]
              temporalCountsMap <- zoo::coredata(temporalCountsMap)
              
              dataTemporalMap@probabilityMap <- temporalMap
              dataTemporalMap@countsMap      <- temporalCountsMap
              dataTemporalMap@dates          <- dates
              
              return(dataTemporalMap)
          }
)

---
title: "EHRtemporalVariability: <br>
Delineating temporal dataset shifts in  <br>
Electronic Health Records"
subtitle: |
  <div style="font-size: 0.95em; font-weight: normal; line-height: 1.4; color: #444;">
  
  Carlos Sáez<sup>1,2</sup>, Alba Gutiérrez-Sacristán<sup>2</sup>, Isaac Kohane<sup>2</sup>, Juan M. García-Gómez<sup>1</sup>, Paul Avillach<sup>2</sup>  
  
  <span style="font-size: 0.9em;">
  <sup>1</sup> Biomedical Data Science Lab, Institute of Information and Communication Technologies,<br>
  Universitat Politècnica de València, Spain  
  <sup>2</sup> Department of Biomedical Informatics, Harvard Medical School, USA  
  </span>
  
  <span style="font-size: 0.9em;">✉️ carsaesi@upv.es</span>
  
  </div>
date: "Apr 27, 2026"
package: "EHRtemporalVariability `r packageVersion('EHRtemporalVariability')`"
bibliography: general-overview.bib
vignette: >
  %\VignetteIndexEntry{EHRtemporalVariability}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
output:
  rmarkdown::html_vignette:
    toc: true
    number_sections: true
link-citations: true
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(
    echo = TRUE,
    warning=FALSE,
    fig.width = 7,
    fig.height = 4,
    out.width = "100%")
```

# Introduction
The `EHRtemporalVariability` package contains functions to delineate temporal dataset shifts in Electronic Health Records through the projection and visualization of dissimilarities among data temporal batches. This is done through the estimation of data statistical distributions over time and their projection in non-parametric statistical manifolds, uncovering the patterns of the data latent temporal variability. Dataset shifts can be explored and identified through visual analytics formats such as Data Temporal heatmaps and Information Geometric Temporal (IGT) plots [@saez_probabilistic_2015; @saez2016applying; @saez2018kinematics]. An additional [EHRtemporalVariability Shiny app](https://github.com/hms-dbmi/EHRtemporalVariability-shiny) can be used to load and explore the package results towards an improved investigation experience and even to allow the use of these functions to those users non-experienced in R coding.

If you use `EHRtemporalVariability` please cite:

<span style="background-color:#eef5ff">Carlos Sáez, Alba Gutiérrez-Sacristán, Isaac Kohane, Juan M García-Gómez, Paul Avillach. EHRtemporalVariability: delineating temporal data-set shifts in Electronic Health Records. GigaScience, Volume 9, Issue 8, August 2020, giaa079. [doi:10.1093/gigascience/giaa079](https://doi.org/10.1093/gigascience/giaa079)</span> [@saez2020]


## Background
Biomedical Data research repositories and property biomedical research databases are becoming bigger both in terms of sample size and collected variables [@Gewin2016; @Andreu-Perez2015]. Two significant reasons behind of this are the widespread adoption of data-sharing initiatives and technological infrastructures, and the continuous and systematic population of those repositories over longer periods of time. However, these two situations can also introduce potential confounding factors in data which may hinder their reuse for research, such as in population research or in statistical and machine learning modeling. Concretely, differences in protocols, populations, or even unexpected biases, either caused by systems or humans, can lead to temporal dataset shifts [@quionero2009dataset; @moreno2012unifying], changes of reference which are reflected in the statistical distributions of data. This temporal variability of data represent a Data Quality (DQ) issue which must be addressed for a reliable data reuse [@saez2016applying; @schlegel2017secondary].

The `EHRtemporalVariability` package has been developed to help preventing this problem.

The tasks that can be performed with `EHRtemporalVariability` package are the following:

  1. Estimate the probability distributions of numerical and coded variables on temporal batches at a yearly, monthly or weekly period.
  2. Plot and explore `data temporal heatmaps` of absolute and relative frequencies of variable values over temporal batches.
  3. Estimate a projection of the variability among data temporal batches, as a non-parametric information geometry embedding of the probabilistic distances among the probability distributions of temporal batches at a number of dimensions specified by the user, allowing for plotting (see next point) or further data analysis methods (such as unsupervised learning of temporal batches).
  4. Plot and explore the projection above through an `Information Geometric Temporal plot`, which helps delineating reference changes in data over time, including abrupt and recurrent changes, conceptually-related time periods (periods with similar data distributions), but also smooth temporal trends.
  5. Additional data pre- and processing options, such as mapping International Classification of Diseases 9th revision (ICD-9) codes to phenotype codes used in Phenome-Wide Association studies (PheWAS), or trimming data temporal maps.

In the following sections the specific functions that can be used to address each of these tasks are presented.

<figure style="text-align: center; margin: 1.5em 0;">
  <img src="rpackageworkflow.png" style="width:60%;">
  <figcaption style="font-size: 0.9em; color: #555;">
    General workflow for using the EHRtemporalVariability R package
  </figcaption>
</figure>

For more information about the methods please check reference [@saez_probabilistic_2015] and the Suplemental Material in [@saez2016applying].

## Installation 
`EHRtemporalVariability` is provided through CRAN and GitHub. To install the CRAN version the user must type the following commands in an R session:

```{r cran, message=FALSE, eval=FALSE, warning=FALSE}
install.packages("EHRtemporalVariability")
library(EHRtemporalVariability)
```

```{r crantrue, echo=FALSE, message=FALSE, eval=TRUE, warning=FALSE}
library(EHRtemporalVariability)
```

The GitHub version of the package will, in general, provide the latest updates before these are commited to the CRAN version. In order to install it, `devtools` package - available in CRAN (https://cran.r-project.org/) - is required. To install `devtools` the user must type the following commands in an R session:

```{r devtls, message=FALSE, eval=FALSE, warning=FALSE}
install.packages("devtools")
library(devtools)
```

Once `devtools` package has been installed the user can install `EHRtemporalVariability` typing the following commands in an R session:

```{r bioC, message=FALSE, eval=FALSE, warning=FALSE}
install_github("hms-dbmi/EHRtemporalVariability")
library( EHRtemporalVariability )
```

## S4 objects 

### `DataTemporalMap`

The `DataTemporalMap` object contains the statistical distributions of data estimated at a specific time period. 

```{r EHRtemporalVariabilityObj1, echo = FALSE, eval = TRUE, warning=FALSE}
githubURL <- "https://github.com/hms-dbmi/EHRtemporalVariability-DataExamples/raw/master/variabilityDemoNHDS.RData"
if (httr::http_error(githubURL)) {
      message("URL not accessible or no internet connection.")
      load(system.file("extdata","variabilityDemoNHDSdiagcode1-phewascode.RData",package="EHRtemporalVariability"))
} else {
    load(url(githubURL))
}
```

```{r variabilityObj2, eval=TRUE, warning=FALSE}
class( probMaps$`diagcode1-phewascode` )
```
<br>
`DataTemporalMap` object is the output of `estimateDataTemporalMap` function. It is used as input for `plotDataTemporalMap` functions.

<br>
<span style="background-color:#eef5ff">Note that objects of this class can be generated automatically by the `estimateDataTemporalMap` function, but its construction and extension is open towards fostering its use through external methods. E.g., one may use additional probability distribution estimation methods, or even contruct compatible `DataTemporalMap` object for other unstructured data shuch as images or free text.</span>

### `IGTProjection`

The `IGTProjection` object contains the projected non-parametric statistical manifold of a `DataTemporalMap` object (also included in the object) estimated in a specific number of dimensions.

```{r igtProjObj, eval=TRUE, warning=FALSE}
class( igtProjs$`diagcode1-phewascode` )
```
<br>
`IGTProjection` object is the output of `estimateIGTProjection` function. It is used as input for `plotIGTProjection` functions.

<br>
<span style="background-color:#eef5ff">Note that objects of this class are generated automatically by the `estimateIGTProjection` function.</span>

# Data pre-processing 

## Load the CSV input file 
The first step consists on read the CSV file that contains the data for the analysis. To do it, the user can apply the `read.csv` function.

The `read.csv` function reads a file in table format and creates a data frame from it, with cases corresponding to lines and variables to fields in the file. It is important to define the class of each column when reading the CSV file. 

An example of how to read the CSV file is shown next:

```{r readCSV, eval=TRUE, warning=FALSE}
dataset <- read.csv2( "http://github.com/hms-dbmi/EHRtemporalVariability-DataExamples/raw/master/nhdsSubset.csv", 
                      sep  = ",",
                      header = TRUE, 
                      na.strings = "", 
                      colClasses = c( "character", "numeric", "factor",
                                      "numeric" , rep( "factor", 22 ) ) )
head( dataset)
```

## Transform the date column in 'Date' R format
The second step will be transform the date column in 'Date' R format. The `EHRtemporalVariability` R package allows the user to do this transformation applying the `formatDate` function.

The `formatDate` function transform the column containing the dates from the given data.frame input, using the following arguments:

  - _input_: a data.frame object with at least one column of dates.
  - _dateColumn_: the name of the column containing the date.
  - _dateFormat_: the date format as standard in R Dates, by default '%y/%m/%d'.

The `formatDate` function output is the same data.frame with the date column transformed.

```{r formatDate2, eval=TRUE, warning=FALSE}
class( dataset$date )
datasetFormatted <- EHRtemporalVariability::formatDate(
              input         = dataset,
              dateColumn    = "date",
              dateFormat = "%y/%m"
             )
head( datasetFormatted )[1:5, 1:5]
class( datasetFormatted$date )
```

## Transform the ICD9-CM into PheWAS codes
Once the CSV file has been readed and the date column transformed, the third step is to transform the ICD9-CM to PheWAS codes if needed (https://phewascatalog.org/) [@denny2013systematic]. 

<span style="background-color:#eef5ff">This step is not mandatory, is an option for those analysis in which ICD9-CM are analyzed and want to be reduced and transformed into PheWAS codes.</span>  

The `EHRtemporalVariability` R package allows the user to do the mapping in an automatic way applying the `icd9toPheWAS` function.

The `icd9toPheWAS` map to PheWAS codes using as input a column containing ICD9-CM codes. This function has as input the following arguments:

  - _data_: a data.frame object with at least one column of ICD9-CM codes that one to be transformed into a PheWAS code.
  - _icd9ColumnName_: the name of the column containing the ICD9-CM.
  - _missingValues_: the value used to determine missing values in the data.frame.
  - _phecodeDescription_: an optional argument to determine if instead to mapping to the PheWAS code, the user is interested in mapping to the PheWAS code description. 
  - _replaceColumn_: an optional argument to determine if you want to replace the ICD9-CM codes column or if you want to generate a new one for the PheWAS codes. 
  - _statistics_: if TRUE, shows the summary of the mapping like the percentage of initial ICD9-CM codes mapped to PheWAS code.
  - _replaceColumn_: whether to replace the original ICD9-CM column with the PheWAS codes or, if FALSE, create a new column with the PheWAS code.


The `icd9toPheWAS` function output is the ICD9-CM column transformed into PheWAS codes. In this specific "NHDS" example we will create a new column with the PheWAS codes that map to the diagcode2 column in the original data.frame. 

```{r icd9toPheWAS, eval=TRUE, message=FALSE, warning=FALSE}
datasetPheWAS <- icd9toPheWAS(data           = datasetFormatted,
                              icd9ColumnName = "diagcode1",
                              phecodeDescription = TRUE,
                              missingValues  = "N/A", 
                              statistics     = TRUE, 
                              replaceColumn  = FALSE)

head( datasetPheWAS[, c( "diagcode1", "diagcode1-phewascode")] )
```

# Data analysis

## Estimate data temporal map
The `estimateDataTemporalMap` function estimates a `DataTemporalMap` object from a data.frame containing individuals in rows and the variables in columns, being one of these columns the analysis date. This function has as input the following arguments:

  - _data_: a data.frame containing as many rows as individuals, and as many columns as the analysis variables plus the individual acqusition date.
  - _dateColumnName_: a string indicating the name of the column in data containing the analysis date variable.
  - _period_ : the period at which to batch data for the analysis from "week", "month" and "year", with "month" as default.

Additionally this function has the following optional arguments:

  - _startDate_: a Date object indicating the date at which to start the analysis. By default the first chronological date in the date column will be selected.
  - _endDate_: a Date object indicating the date at which to end the analysis. By default the last chronological date in the date column will be selected. 
  - _supports_: a List of objects containing the support of the data distributions for each variable, in classes `numeric`, `integer`, `character`, or `factor` (accordingly to the variable type), and where the name of the list element must correspond to the column name of its variable. If not provided it is automatically estimated from data.
  - _numericVariablesBins_: the number of bins at which to define the frequency histogram for numerical variables. By default is set to 100.
  - _numericSmoothing_: a logical value indicating whether a Kernel Density Estimation smoothing (Gaussian kernel, default bandwith) is to be applied on numerical variables (the default) or a traditional histogram instead.
  - _dateGapsSmoothing_: a logical value indicating whether a linear smoothing is applied to those time batches without data, by default gaps are filled with NAs.

The `estimateDataTemporalMap` function output is a `DataTemporalMap` object or a list of `DataTemporalMap` objects depending on the number of analysis variables.

```{r estimateDataTemporalMap, eval=FALSE, warning=FALSE}
probMaps <- estimateDataTemporalMap(data           = datasetPheWAS, 
                                    dateColumnName = "date", 
                                    period         = "month")
```

In the previous specific example, nhds data frame is used as input, with the `date` column previously formated to `Date` format. The `estimateDataTemporalMap` function has been applied to the X variables present in the initial data set. As a result, a list of X `DataTemporalMap` objects is obtained.

```{r estimateDataTemporalMapOutput, eval=TRUE, warning=FALSE}
class( probMaps )
class( probMaps[[ 1 ]] )
```

Variable supports can be set manually for all or some of the variables using the `support` parameter. The support of those variables not present in the `support` parameter will be estimated automatically as when the parameter is not passed.

```{r estimateDataTemporalMapSupport, eval=FALSE, warning=FALSE}
supports <- vector("list",2)
names(supports) <- c("age","diagcode1")
supports[[1]] <- 1:18
supports[[2]] <- c("V3000","042--","07999","1550-","2252-")
probMapsWithSupports <- estimateDataTemporalMap(data           = datasetPheWAS, 
                                    dateColumnName = "date", 
                                    period         = "month",
                                    supports       = supports)
```

## Trim a DataTemporalMap object
Additionally, the `EHRtemporalVariability` R package contains a function that allows the user to trim any `DataTemporalMap` object according to a start and end date. 

The `trimDataTemporalMap` function needs as input the following arguments:

  - _dataTemporalMap_: a `DataTemporalMap` object.
  - _startDate_: a date indicating the start date to trim from.
  - _endDate_: a date indicating the end date to trim from.


The `trimDataTemporalMap` function output is a new `DataTemporalMap` object.

```{r trimDataTemporalMap, eval=TRUE, warning=FALSE}
class( probMaps[[1]] )
probMapTrimmed <- trimDataTemporalMap( 
                        dataTemporalMap = probMaps[[1]],
                        startDate       = "2005-01-01",
                        endDate         = "2008-12-01"
                                      )
class( probMapTrimmed )
```


## Estimate IGT projections
The `estimateIGTProjection` function estimates a `IGTProjection` object from a `DataTemporalMap` object. This function has as input the following arguments:

  - _dataTemporalMap_: a `DataTemporalMap` object.
  - _dimensions_: integer value indicating the number of dimensions for the projection.
  - _startDate_: a Date object indicating the date at which to start the analysis, in case of being different from the first chronological date in the date column (the default).
  - _endDate_: a Date object indicating the date at which to end the analysis, in case of being different from the last chronological date in the date column (the default).
  - _embeddingType_: the type of embedding in order to obtain the non-parametric Statistical Manifold, from `classicalmds` and `nonmetricmds`, with `classicalmds` as default. 


The `estimateIGTProjection` function output is a `IGTProjection` object.

```{r estimateIGTProjection, eval=TRUE, warning=FALSE}
igtProj <- estimateIGTProjection( dataTemporalMap = probMaps[[1]], 
                                  dimensions      = 2, 
                                  startDate       = "2000-01-01", 
                                  endDate         = "2010-12-31")
```

The `estimateIGTProjection` function can be applied to one `DataTemporalMap` object. As a result, an `IGTProjection` object is obtained.

```{r estimateIGTProjectionOutput, eval=TRUE, warning=FALSE}
class( igtProj )
```

The `sapply` function can be used to apply the `estimateIGTProjection` function to the output from `estimateDataTemporalMap` function including more than one variable, which result is a list of `DataTemporalMap` objects, as follows:

```{r sapplyestimateIGTProjection, eval=FALSE, warning=FALSE}
igtProjs <- sapply ( probMaps, estimateIGTProjection )
names( igtProjs ) <- names( probMaps )
```

# Data visualization
`EHRtemporalVariability` offers two different options to visualize the results, heatmaps and Information Geometric Temporal (IGT) plots. An special focus is made on visualization colors. 

<span style="background-color:#eef5ff">The default "Spectral" palette shows a color temperature scheme from blue, through yellow, to red. The four remaining options are better suited for those with colorblindness, including "Viridis", "Magma", and their reversed versions "Viridis-reversed" and "Magma-reversed".</span>

## Plot data temporal maps
The `plotDataTemporalMap` function returns a heatmap or time series plot.  This function has as input the following arguments:

  - _dataTemporalMap_: the `DataTemporalMap` object.
  - _absolute_: inidicates if the heatmap frequency values are absolute or relative. By default FALSE.  
  - _startValue_: indicates the first value to display in the heatmap. By default 1. 
  - _endValue_: indicates the last value to display in the heatmap. By default the last value of the DataTemporalMap object.  
  - _startDate_: a Date object indicating the first date to be displayed in the heatmap. By default the first date of the DataTemporalMap object. 
  - _endDate_: a Date object indicating the last date to be displayed in the heatmap. By default the last date of the DataTemporalMap object.
  - _sortingMethod_: the method to sort data in the Y axis of the heatmap from "frequency" and "alphabetical", with "frequency" as default.
  - _colorPalette_: the color palette to be used. 
  - _mode_: whether to plot as a heatmap (by default) or time series. 

To illustrate the next examples we load the example .Rdata file, which contains the results from analyzing the complete "NHDS" dataset.

```{r loadExampleFile, eval=TRUE, warning=FALSE}
githubURL <- "https://github.com/hms-dbmi/EHRtemporalVariability-DataExamples/raw/master/variabilityDemoNHDS.RData"
load(url(githubURL))
```

### Heatmap plot

```{r plotHeatmap, eval=TRUE, warning=FALSE}
plotDataTemporalMap(
    dataTemporalMap =  probMaps[["diagcode1-phewascode"]],
    startValue = 2,
    endValue = 20,
    colorPalette    = "Spectral")
```

### Time series plot

```{r plotSeries, eval=TRUE, warning=FALSE}
plotDataTemporalMap(
    dataTemporalMap =  probMaps[["diagcode1-phewascode"]],
    startValue = 2,
    endValue = 10,
    mode = "series")
```

## Plot IGT projections
The `plotIGTProjection` function returns an interactive Information Geometric Temporal (IGT) plot from an `IGTProjection` object. This function has as input the following arguments:

  - _igtProjection_: the `IGTProjection` object.
  - _dimensions_: the number of dimensions of the plot (2 or 3).
   - _startDate_: a Date object indicating the first date to be displayed in the IGT plot. By default the first date of the IGTProjection object. 
  - _endDate_: a Date object indicating the last date to be displayed in the IGT plot By default the last date of the IGTProjection object.
  - _colorPalette_: the color palette to be used.
  - _trajectory_:  whether to show an estimated trajectory of the information evolution over time. 

```{r plotIGTprojection, eval=TRUE, warning=FALSE}
plotIGTProjection( 
    igtProjection   =  igtProjs[["diagcode1-phewascode"]],
    colorPalette    = "Spectral", 
    dimensions      = 2)
```

An IGT plot visualizes the variability among time batches in a data repository in a 2D or 3D plot. Time batches are positioned as points where the distance between them represents the probabilistic distance between their distributions (currently Jensen-Shannon distance, more distances will be supported in the future). 

<br>
To track the temporal evolution, temporal batches are labeled to show their date and colored according to their season or period, according to the analysis period, as follows. If period=="year" the label is "yy" (2 digit year) and the color is according to year. If period=="month" the label is "yym" (yy + abbreviatted month) and the color is according to the season (yearly). If period=="week" the label is "yymmw" (yym + ISO week number in 1-2 digit) and the color is according to the season (yearly). 


| Month | Abbreviaton | 
| ------|-------------|
| January | J         |
| February | F         |
| March | M         |
| April | a         |
| May | m         |
| June | j         |
| July | x         |
| August | a         |
| September | S         |
| October | O         |
| November | N         |
| December | D         |

### Visualize temporal trajectory {#trajectory}

The `plotIGTProjection` function allows overlying to both 2D and 3D IGT plots an smoothed trajectory of the information evolution over time, which is calculated with smoothed splines. To plot the trajectory set the _trajectory_ parameter to `TRUE`.

```{r plotIGTprojectionTrajectory, eval=TRUE, warning=FALSE}
plotIGTProjection( 
    igtProjection   =  igtProjs[["diagcode1-phewascode"]],
    colorPalette    = "Spectral", 
    dimensions      = 2,
    trajectory      = TRUE)
```

## Export data for the Shiny app dashboard

The [EHRtemporalVariability Shiny app](https://github.com/hms-dbmi/EHRtemporalVariability-shiny) allows loading your own .csv file through simple configuration steps but, however, with limited data pre-processing. Consequently, users can export the DataTemporalHeatmaps and IGTplots generated from the R package as an .RData file for their exporation through the interactive Shiny app dashboard. The export is done as follows (note that both DataTemporalHeatmaps and IGTplots must be lists of same size where the names of their items correspond to the variable names):

```{r saveRData, eval=FALSE, warning=FALSE}
names( probMaps )
names( igtProjs )
save(probMaps, igtProjs, file = "myExport.RData")
```

# Interpretation of temporal changes in IGT projections

## Types of temporal changes

According to the layout of time batches in IGT projections we define the following four types of temporal changes (quoting text from our previous publication [@saez2018kinematics]):

* _Trend:_ Continuous and smooth change in the probability distributions of time batches over time, along the full-time period, or within a sub-period. Trends can be linear or, more generally, curved. Trends are represented in the IGT projection as a continuous flow of time batches through a time-related direction related.

* _Abrupt change:_ A sudden change in probability distributions at a specific time point, leading to a new data inherent concept which is maintained afterwards. Abrupt changes are represented in the IGT projection as a gap between two groups of continuous time batches. Multiple abrupt changes can occur in a data repository, splitting the dataset into multiple clusters of time batches (see the definition of temporal subgroups). A single time batch could be abruptly separated from the rest, generally due to some specific context in its data (e.g., transient states, incomplete batches), in that case, we will talk about an _outlier batch_.

* _Temporal subgroups:_ Conceptually related groups of time periods at which probability distributions are similar within a group, but dissimilar between groups, i.e., forming clusters of time batches. Abrupt changes do generally split data into temporal subgroups. A consecutive time flow between batches at two temporal subgroups would indicate a recurrent behaviour. An outlier batch will not be considered within any subgroup.

* _Seasonality:_ Repetition of some change patterns at a specific time period throughout the IGT projection. Seasonality should be represented in the IGT projection as repetitive cycles over the general temporal flow. We could find local seasonality, within a specific time period, or global seasonality, along the full study period. Global seasonality should be maintained even across multiple temporal subgroups, e.g., in a data repository which is partitioned on various temporal subgroups, a global yearly variation should be maintained across the different subgroups.

## Example

To ilulstrate this example we will refer to the IGT plot for variable `diagcode1-phewascode` shown in the [temporal trajectory visualization](#trajectory) section. In that IGT plot we can find the following changes.

* _Trend:_ A main trend through the entire period of study can be found across dimension D1, where there is a continuous flow of time batches. This is possibly associated to smooth population changes over time, e.g, including changes in life expectancy.

* _Seasonality:_ There is an evident yearly seasonality through the entire period of study layering across dimension D2, and highlighted by the batches coloring scheme and cycles on the trajectory. This is likely associated to the yearly seasonality of diseases.

* _Abrupt change:_ There is an abrupt change which splits the main trend between 2007 and 2008, with an apparent minor transient start in November 2007. This is possibly related to changes in the hospitals providing data to the NHDS as well as yearly ICD-9-CM updates in 2007. Another minor abrupt change shows as well between 2004 and 2005, with possible equivalent causes.

* _Temporal subgroups:_ The abrupt changes described before define as well distinct temporal subgroups. One possibility to validate them is using clustering algorithms. As an example, we can apply the DBSCAN clustering algorithm to the IGT projection points, to obtain the following results.

```{r dbscan, message=FALSE, eval=FALSE, warning=FALSE}
install.packages("dbscan")
library(dbscan)
```

```{r dbscantrue, echo=FALSE, message=FALSE, eval=requireNamespace("dbscan", quietly=TRUE), warning=FALSE}
library(dbscan)
```

```{r temporalSubgroupsClustering, eval=requireNamespace("dbscan", quietly=TRUE), warning=FALSE}
# We set the minimum number of batches in a subgroup as 2 
# We set eps based on the knee of the following KNNdistplot, at around 0.023
# kNNdistplot(igtProj@projection, k = 2, all = FALSE)
igtProj = igtProjs[["diagcode1-phewascode"]]
# We select the 2 first dimensions for consistency with the IGT plot examples above
dbscanResults <- dbscan::dbscan(igtProj@projection[,c(1,2)], eps = 0.023, minPts = 2)
clusterNames  <- vector(mode = "character", length = 10)
clusterNames[dbscanResults$cluster == 0] <- "Outlier batches"
clusterNames[! dbscanResults$cluster == 0] <- paste("Temporal subgroup",dbscanResults$cluster[! dbscanResults$cluster == 0])
plotly::plot_ly(x = igtProj@projection[,1], y = igtProj@projection[,2],
              color = as.factor(clusterNames),
              type = "scatter", mode = "markers",
              text = paste0("Date: ",igtProj@dataTemporalMap@dates)) %>%
              plotly::config(displaylogo = FALSE)
```

<span style="background-color:#eef5ff">Note that the sensitivity of the cluster validation can be refined by using IGT projections at 3 or more dimensions, modifying the DBSCAN parameters, or using other clustering algorithms.</span>

# Summary of `EHRtemporalVariability` available functions

| Input Object    | `EHRtemporalVariability` function | Output Generated                                |
| ----------------|-----------------------|---------------------------------------------------|
| data.frame       | `formatDate`             | Given a data.frame object with a column of dates in 'character' format, it generates a new data.frame object with the dates transformed into "Date" R format.|
| data.frame       | `icd9toPheWAS`        | Given a data.frame object with a column of ICD9-CM codes, it generates a new data.frame object with the ICD9-CM codes transformed into PheWAS codes |
| data.frame       | `estimateDataTemporalMap` | Given a data.frame object containing individuals in rows and the variables in columns, it generates a `DataTemporalMap` object or a list of DataTemporalMap objects depending on the number of analysis variables |
| `DataTemporalMap`| `trimDataTemporalMap` | Given a `DataTemporalMap` object, it generates a trimmed `DataTemporalMap` object |
| `DataTemporalMap`| `estimateIGTProjection` | Given a `DataTemporalMap` object, it generates a `IGTProjection` object |
| `DataTemporalMap`| `plotDataTemporalMap` | Given a `DataTemporalMap` object, it generates an interactive heatmap |
| `IGTProjection`| `plotIGTProjection` | Given an `IGTProjection` object, it generates an interactive Information Geometric Temporal (IGT) plot |
| `IGTProjection`| `estimateIGTTrajectory` | Given an`IGTProjection` object, it estimates a trajectory of the information temporal evolution in a IGT projection by fitting a cubic smoothing spline |

: (\#tab:viz-opt) Functions in `EHRtemporalVariability` R package



# Bibliography

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/allGenerics.R, R/allMethods.R
\name{plotDataTemporalMap}
\alias{plotDataTemporalMap}
\alias{plotDataTemporalMap,DataTemporalMap-method}
\title{Data Temporal heatmap}
\usage{
plotDataTemporalMap(
  dataTemporalMap,
  absolute = FALSE,
  startValue = 1,
  endValue = ncol(dataTemporalMap@probabilityMap),
  startDate = min(dataTemporalMap@dates),
  endDate = max(dataTemporalMap@dates),
  sortingMethod = "frequency",
  colorPalette = "Spectral",
  mode = "heatmap"
)

\S4method{plotDataTemporalMap}{DataTemporalMap}(
  dataTemporalMap,
  absolute = FALSE,
  startValue = 1,
  endValue = ncol(dataTemporalMap@probabilityMap),
  startDate = min(dataTemporalMap@dates),
  endDate = max(dataTemporalMap@dates),
  sortingMethod = "frequency",
  colorPalette = "Spectral",
  mode = "heatmap"
)
}
\arguments{
\item{dataTemporalMap}{of class \code{DataTemporalMap}}

\item{absolute}{indicates if the heatmap frequency values are absolute or relative. 
By default \code{FALSE}.}

\item{startValue}{indicates the first value to display in the heatmap.
By default 1.}

\item{endValue}{indicates the last value to display in the heatmap.
By default the last value of the \code{DataTemporalMap} object.}

\item{startDate}{a Date object indicating the first date to be displayed in the heatmap. 
By default the first date of the \code{DataTemporalMap} object.}

\item{endDate}{a Date object indicating the last date to be displayed in the heatmap. 
By default the last date of the \code{DataTemporalMap} object.}

\item{sortingMethod}{the method to sort data in the Y axis of the heatmap from "frequency" and 
"alphabetical", with "frequency" as default.}

\item{colorPalette}{color palette to be used. The default "Spectral" palette shows a 
color temperature scheme from blue, through yellow, to red (see "Spectral" palette in
 RColorBrewer package). The four remaining options are better suited for those with 
 colorblindness, including "Viridis", "Magma", and their reversed versions 
 "Viridis-reversed" and "Magma-reversed" (see "Viridis" and "Magma" palettes in the
 Viridis package).}

\item{mode}{indicates the plot mode as a 'heatmap' (default) or 'series'.
The other config parameters for the heatmap plot also apply for the series plot.}
}
\value{
A plot object based on the \code{plotly} package.
}
\description{
Plots a Data Temporal heatmap from an \code{DataTemporalMap} object.
}
\examples{
load(system.file("extdata",
                 "variabilityDemoNHDSdiagcode1-phewascode.RData",
                  package="EHRtemporalVariability"))

p <- plotDataTemporalMap(dataTemporalMap =  probMaps[[1]],
                    colorPalette    = "Spectral",
                    startValue = 2,
                    endValue = 40)
p

p <- plotDataTemporalMap(dataTemporalMap =  probMaps[[1]],
                    colorPalette    = "Spectral",
                    startValue = 2,
                    endValue = 40,
                    mode = "series")
p

\dontrun{

# For additional and larger examples download the following .Rdata file:

gitHubUrl  <- 'http://github.com/'
gitHubPath <- 'hms-dbmi/EHRtemporalVariability-DataExamples/'
gitHubFile <- 'raw/master/variabilityDemoNHDS.RData'
inputFile  <-  paste0(gitHubUrl, gitHubPath, gitHubFile)

load(url(inputFile))
plotDataTemporalMap(probMaps$`diagcode1-phewascode`, startValue = 2, endValue = 40)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/allGenerics.R, R/allMethods.R
\name{trimDataTemporalMap}
\alias{trimDataTemporalMap}
\alias{trimDataTemporalMap,DataTemporalMap-method}
\title{Trims a \code{DataTemporalMap}}
\usage{
trimDataTemporalMap(
  dataTemporalMap,
  startDate = min(dataTemporalMap@dates),
  endDate = max(dataTemporalMap@dates)
)

\S4method{trimDataTemporalMap}{DataTemporalMap}(
  dataTemporalMap,
  startDate = min(dataTemporalMap@dates),
  endDate = max(dataTemporalMap@dates)
)
}
\arguments{
\item{dataTemporalMap}{of class \code{DataTemporalMap}.}

\item{startDate}{\code{Date} indicating the start date to trim from.}

\item{endDate}{\code{Date} indicating the end date to trim to.}
}
\value{
A \code{DataTemporalMap} object between the specified dates.
}
\description{
Trims a \code{DataTemporalMap} object between an start and end date. If one is not specified it takes 
as default the first/last chronological date in the input \code{DataTemporalMap}.
}
\examples{
load(system.file("extdata",
                 "variabilityDemoNHDSdiagcode1-phewascode.RData",
                  package="EHRtemporalVariability"))

probMapTrimmed <- trimDataTemporalMap( 
                         dataTemporalMap = probMaps[[1]],
                         startDate       = "2005-01-01",
                         endDate         = "2008-12-01"
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formatDate.R
\name{formatDate}
\alias{formatDate}
\title{Function to transform dates into "Date" R format}
\usage{
formatDate(input, dateColumn, dateFormat = "\%y/\%m/\%d", verbose = FALSE)
}
\arguments{
\item{input}{A \code{data.frame} object with at least one column of dates.}

\item{dateColumn}{The name of the column containing the date.}

\item{dateFormat}{By default \code{'\%y/\%m/\%d'}. Change it to the specific structure of your date format.}

\item{verbose}{By default \code{FALSE}. Change it to \code{TRUE} to get an on-time log from the function.}
}
\value{
An object of class \code{data.frame} with the date column transform into 'Date' R class.
}
\description{
Given a \code{data.frame} object with a column of dates in 'character' format, 
it generates a new \code{data.frame} object with the dates transformed into 
"Date" R format.
}
\examples{

dataset <- read.csv2(xzfile(system.file("extdata",
                                   "nhdsSubset.csv.xz",
                                   package="EHRtemporalVariability")), 
                     sep  = ",",
                     header = TRUE, 
                     na.strings = "", 
                     colClasses = c( "character", "numeric", "factor",
                                     "numeric" , rep( "factor", 22 ) ) )
                     
datasetFormatted <- formatDate( 
              input         = dataset, 
              dateColumn    = "date", 
              dateFormat    = "\%y/\%m",
             )
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/allClasses.R
\docType{class}
\name{IGTProjection-class}
\alias{IGTProjection-class}
\alias{IGTProjection}
\alias{IGTProjection,IGTProjection-class}
\title{Class IGTProjection}
\value{
A \code{IGTProjection} object.
}
\description{
Class \code{IGTProjection} object contains the estimated Information Geometric Temporal plot projection
of a \code{DataTemporalMap} object.
}
\details{
Objects of this class are generated automatically by the \code{estimateIGTProjection} function, and can
be passed to the \code{plotIGTProjection} function for plotting the Information Geometric Temporal plot.
}
\section{Slots}{

\describe{
\item{\code{dataTemporalMap}}{the \code{DataTemporalMap} object from which the projection was estimated}

\item{\code{projection}}{d-by-c \code{matrix} of the IGT projection for d temporal batches in c dimensions calculated from the \code{estimateIGTProjection} function}

\item{\code{embeddingType}}{embedding function used for the IGT projection calculation in the \code{estimateIGTProjection} function}

\item{\code{stress}}{stress achieved in the lower-dimensional embedding, depending on the embedding type (e.g., classicalmds will return 1-GOF as returned by stats::cmdscale function, and nonmetricmds will return final stress in percent, as returned by the MASS::isoMDS function)}
}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/estimateIGTTrajectory.R
\name{estimateIGTTrajectory}
\alias{estimateIGTTrajectory}
\title{Estimates a trajectory of the information temporal evolution in a IGT projection by fitting a cubic smoothing spline}
\usage{
estimateIGTTrajectory(igtProjection, nPoints = NULL)
}
\arguments{
\item{igtProjection}{of class \code{IGTProjection}.}

\item{nPoints}{the number of points to fit within the IGT projection range. By default 10x the number of time batches, what shows a high resolution trajectory.}
}
\value{
A list containing a \code{data.frame} of the estimated trajectory points, the estimated date for each point, and the fitted trajectory function as \code{smooth.spline} objects.
}
\description{
Estimates a \code{DataTemporalMap} from a \code{data.frame} containing individuals in rows and the 
variables in columns, being one of these columns the analysis date (typically the acquisition date). 
Will return a \code{DataTemporalMap} object or a \code{list} of \code{DataTemporalMap} objects 
depending on the number of analysis variables.
}
\examples{
load(system.file("extdata",
                 "variabilityDemoNHDSdiagcode1-phewascode.RData",
                  package="EHRtemporalVariability"))

igtTrajectory <- estimateIGTTrajectory( igtProjection   =  igtProjs[[1]] )
igtTrajectory$points

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/allGenerics.R, R/allMethods.R
\name{plotIGTProjection}
\alias{plotIGTProjection}
\alias{plotIGTProjection,IGTProjection-method}
\title{Information Geometric Temporal plot}
\usage{
plotIGTProjection(
  igtProjection,
  dimensions = 3,
  startDate = min(igtProjection@dataTemporalMap@dates),
  endDate = max(igtProjection@dataTemporalMap@dates),
  colorPalette = "Spectral",
  trajectory = FALSE
)

\S4method{plotIGTProjection}{IGTProjection}(
  igtProjection,
  dimensions = 3,
  startDate = min(igtProjection@dataTemporalMap@dates),
  endDate = max(igtProjection@dataTemporalMap@dates),
  colorPalette = "Spectral",
  trajectory = FALSE
)
}
\arguments{
\item{igtProjection}{of class \code{IGTProjection}}

\item{dimensions}{number of dimensions of the plot, 2 or 3 (3 by default)}

\item{startDate}{a Date object indicating the first date to be displayed in the IGT plot. 
By default the first date of the \code{IGTProjection} object.}

\item{endDate}{a Date object indicating the last date to be displayed in the IGT plot 
By default the last date of the \code{IGTProjection} object.}

\item{colorPalette}{color palette to be used. The default "Spectral" palette shows a color temperature 
scheme from blue, through yellow, to red (see "Spectral" palette in RColorBrewer package). 
The four remaining options are better suited for those with colorblindness, including "Viridis", 
"Magma", and their reversed versions "Viridis-reversed" and "Magma-reversed" (see "Viridis" and 
"Magma" palettes in the Viridis package).}

\item{trajectory}{whether to show an estimated trajectory of the information evolution over time. 
By default \code{FALSE}.}
}
\value{
A plot object based on the \code{plotly} package.
}
\description{
Plots an interactive Information Geometric Temporal (IGT) plot from an \code{IGTProjection} object.
An IGT plot visualizes the variability among time batches in a data repository in a 2D or 3D plot.
Time batches are positioned as points where the distance between them represents the probabilistic 
distance between their distributions (currently Jensen-Shannon distance, more distances will be 
supported in the future).
To track the temporal evolution, temporal batches are labeled to show their date and 
colored according to their season or period, according to the analysis period, as follows.
If period=="year" the label is "yy" (2 digit year) and the color is according to year.
If period=="month" the label is "yym" (yy + abbreviated month*) and the color is according 
to the season (yearly).
If period=="week" the label is "yymmw" (yym + ISO week number in 1-2 digit) and the color is 
according to the season (yearly). An estimated smoothed trajectory of the information evolution
over time can be shown using the optional "trajectory" parameter.
*Month abbreviations: \{'J', 'F', 'M', 'A', 'm', 'j', 'x', 'a', 'S', 'O', 'N', 'D'\}.
}
\details{
Note that since the projection is based on multidimensional scaling, a 2 dimensional 
projection entails a loss of information compared to a 3 dimensional projection. E.g., periodic 
variability components such as seasonal effect can be hindered by an abrupt change or a general trend.
}
\examples{
load(system.file("extdata",
                 "variabilityDemoNHDSdiagcode1-phewascode.RData",
                  package="EHRtemporalVariability"))

p <- plotIGTProjection( igtProjection   =  igtProjs[[1]],
                        colorPalette    = "Spectral",
                        dimensions      = 2)
p

\dontrun{

# For additional and larger examples download the following .Rdata file:

gitHubUrl  <- 'http://github.com/'
gitHubPath <- 'hms-dbmi/EHRtemporalVariability-DataExamples/'
gitHubFile <- 'raw/master/variabilityDemoNHDS.RData'
inputFile  <-  paste0(gitHubUrl, gitHubPath, gitHubFile)

load(url(inputFile)) 
plotIGTProjection(igtProjs$`diagcode1-phewascode`, dimensions = 3)
} 
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/icd9toPheWAS.R
\name{icd9toPheWAS}
\alias{icd9toPheWAS}
\title{Function to transform ICD9-CM codification into PheWAS code}
\usage{
icd9toPheWAS(
  data,
  icd9ColumnName,
  missingValues = "NA",
  phecodeDescription = FALSE,
  statistics = FALSE,
  replaceColumn = TRUE,
  verbose = FALSE
)
}
\arguments{
\item{data}{A \code{data.frame} object with at least one column of ICD9-CM codes that
one to be transformed into a PheWAS code.}

\item{icd9ColumnName}{The name of the column containing the ICD9-CM.}

\item{missingValues}{The value used to determine missing values in the data.frame.}

\item{phecodeDescription}{By default \code{FALSE}. Change it to \code{TRUE} to map
to the PheWAS code description instead to the PheWAS numeric code.}

\item{statistics}{By default \code{FALSE}. Change it to \code{TRUE} to show the 
summary of the mapping like the percentage of initial ICD9-CM codes mapped to PheWAS code.}

\item{replaceColumn}{By default \code{TRUE}. Change it to \code{FALSE} in order to create a
new column with the PheWAS code maintaining the ICD9-CM code.}

\item{verbose}{By default \code{FALSE}. Change it to \code{TRUE} to get an
on-time log from the function.}
}
\value{
An object of class \code{data.frame} with the ICD9-CM column transform into
PheWAS codes.
}
\description{
Given a  \code{data.frame} object with a column of ICD9-CM codes, it generates a 
new \code{data.frame} object with the ICD9-CM codes transformed into PheWAS codes.
}
\examples{
dataset <- read.csv2(xzfile(system.file("extdata",
                                   "nhdsSubset.csv.xz",
                                   package="EHRtemporalVariability")), 
                     sep  = ",",
                     header = TRUE, 
                     na.strings = "", 
                     colClasses = c( "character", "numeric", "factor",
                                     "numeric" , rep( "factor", 22 ) ) )

datasetPheWAS <- icd9toPheWAS( data           = dataset,
                              icd9ColumnName  = "diagcode1", 
                              missingValues   = "N/A", 
                              replaceColumn   = TRUE, 
                              statistics      = TRUE 
                              )
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/estimateDataTemporalMap.R
\name{estimateDataTemporalMap}
\alias{estimateDataTemporalMap}
\title{Estimates \code{DataTemporalMap} objects from raw data}
\usage{
estimateDataTemporalMap(
  data = NULL,
  dateColumnName = NULL,
  period = "month",
  startDate = NULL,
  endDate = NULL,
  supports = NULL,
  numericVariablesBins = 100,
  numericSmoothing = TRUE,
  dateGapsSmoothing = FALSE,
  verbose = FALSE
)
}
\arguments{
\item{data}{a \code{data.frame} containing as many rows as individuals, and as many columns as the 
analysis variables plus the individual acquisition date.}

\item{dateColumnName}{a string indicating the name of the column in \code{data} containing the 
analysis date variable.}

\item{period}{the period at which to batch data for the analysis from "week", "month" and "year", 
with "month" as default.}

\item{startDate}{a Date object indicating the date at which to start the analysis, in case of being 
different from the first chronological date in the date column (the default).}

\item{endDate}{a Date object indicating the date at which to end the analysis, in case of being 
different from the last chronological date in the date column (the default).}

\item{supports}{a List of objects containing the support of the data distributions for each variable, 
in classes \code{numeric}, \code{integer}, \code{character}, or \code{factor} (accordingly to the 
variable type), and where the name of the list element must correspond to the column name of its 
variable. If not provided it is automatically estimated from data.}

\item{numericVariablesBins}{the number of bins at which to define the frequency/density histogram 
for numerical variables when their support is not provided, 100 as default.}

\item{numericSmoothing}{a logical value indicating whether a Kernel Density Estimation smoothing 
(Gaussian kernel, default bandwidth) is to be applied on numerical variables (the default) or a 
traditional histogram instead. See ?density for further details.}

\item{dateGapsSmoothing}{a logical value indicating whether a linear smoothing is applied to those 
time batches without data, by default gaps are filled with NAs.}

\item{verbose}{By default \code{FALSE}. Change it to \code{TRUE} to get an on-time log from the 
function.}
}
\value{
A \code{DataTemporalMap} object.
}
\description{
Estimates a \code{DataTemporalMap} from a \code{data.frame} containing individuals in rows and the 
variables in columns, being one of these columns the analysis date (typically the acquisition date). 
Will return a \code{DataTemporalMap} object or a \code{list} of \code{DataTemporalMap} objects 
depending on the number of analysis variables.
}
\examples{
#Load the file 
dataset <- read.csv2(xzfile(system.file("extdata",
                                   "nhdsSubset.csv.xz",
                                   package="EHRtemporalVariability")), 
                     sep  = ",",
                     header = TRUE, 
                     na.strings = "", 
                     colClasses = c( "character", "numeric", "factor",
                                     "numeric" , rep( "factor", 22 ) ) )
#Format the date
datasetFormatted <- EHRtemporalVariability::formatDate( input         = dataset,
                                            dateColumn    = "date",
                                            dateFormat    = "\%y/\%m")

#Apply the estimateDataTemporalMap
probMaps <- estimateDataTemporalMap( data           = datasetFormatted, 
                                     dateColumnName = "date", 
                                     period         = "month")
\dontrun{

For a larger example download the following .csv dataset and continue the steps as above:

gitHubUrl  <- 'http://github.com/'
gitHubPath <- 'hms-dbmi/EHRtemporalVariability-DataExamples/'
gitHubFile <- 'raw/master/nhdsSubset.csv'
inputFile  <-  paste0(gitHubUrl, gitHubPath, gitHubFile)

dataset <- read.csv2( inputFile, 
                     sep  = ",",
                     header = TRUE, 
                     na.strings = "", 
                     colClasses = c( "character", "numeric", "factor",
                                     "numeric" , rep( "factor", 22 ) ) ) 
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/allGenerics.R, R/allMethods.R
\name{estimateIGTProjection}
\alias{estimateIGTProjection}
\alias{estimateIGTProjection,IGTProjection-method}
\alias{estimateIGTProjection,DataTemporalMap-method}
\title{Estimates an Information Geometric Temporal plot projection}
\usage{
estimateIGTProjection(
  dataTemporalMap,
  dimensions = 3,
  startDate = NULL,
  endDate = NULL,
  embeddingType = "classicalmds"
)

\S4method{estimateIGTProjection}{DataTemporalMap}(
  dataTemporalMap,
  dimensions = 3,
  startDate = NULL,
  endDate = NULL,
  embeddingType = "classicalmds"
)
}
\arguments{
\item{dataTemporalMap}{of class \code{DataTemporalMap} object.}

\item{dimensions}{\code{numeric} integer value indicating the number of dimensions 
for the projection.}

\item{startDate}{a Date object indicating the date at which to start the analysis, 
in case of being different from the first chronological date in the date column 
(the default).}

\item{endDate}{a Date object indicating the date at which to end the analysis, 
in case of being different from the last chronological date in the date column 
(the default).}

\item{embeddingType}{the type of embedding to apply to the dissimilarity matrix of time batches
in order to obtain the non-parametric Statistical Manifold, from "classicalmds" and "nonmetricmds", 
with "classicalmds" as default. "classicalmds" uses the base R stats::cmdscale function, while "nonmetricmds"
uses the MASS:isoMDS function. The returned stress format will depend on the selected embedding type:
"classicalmds" returns 1-GOF as returned by stats::cmdscale function, "nonmetricmds" returns the final stress
in percent, as returned by the MASS::isoMDS function}
}
\value{
An \code{IGTProjection} object containing the projected coordinates of each
temporal batch in the embedded non-parametric Statistical Manifold, as well as the
embedding stress according to the embeddingType.
}
\description{
Estimates an \code{IGTProjection} object from a \code{DataTemporalMap} object.
}
\examples{
load(system.file("extdata",
                 "variabilityDemoNHDSdiagcode1-phewascode.RData",
                  package="EHRtemporalVariability"))
igtProj <- estimateIGTProjection( dataTemporalMap = probMaps$`diagcode1-phewascode`, 
dimensions      = 3, 
startDate       = "2000-01-01", 
endDate         = "2010-12-31")

\dontrun{

# For additional and larger examples download the following .Rdata file:

gitHubUrl  <- 'http://github.com/'
gitHubPath <- 'hms-dbmi/EHRtemporalVariability-DataExamples/'
gitHubFile <- 'raw/master/variabilityDemoNHDS.RData'
inputFile  <-  paste0(gitHubUrl, gitHubPath, gitHubFile)

load(url(inputFile))
igtProj <- estimateIGTProjection( dataTemporalMap = probMaps[[1]], 
dimensions      = 3, 
startDate       = "2000-01-01", 
endDate         = "2010-12-31")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/allClasses.R
\docType{class}
\name{DataTemporalMap-class}
\alias{DataTemporalMap-class}
\alias{DataTemporalMap}
\alias{DataTemporalMap,DataTemporalMap-class}
\title{Class DataTemporalMap}
\value{
A \code{DataTemporalMap} object.
}
\description{
Class \code{DataTemporalMap} object contains the statistical distributions of data estimated at a 
specific time period. Both relative and absolute frequencies are included.
}
\details{
Objects of this class are generated automatically by the \code{estimateDataTemporalMap} function, 
but its construction and extension is open towards fostering its use through external methods. 
E.g., one may use additional probability distribution estimation methods, or even construct 
compatible DataTemporalMaps for other unstructured data such as images or free text.
}
\section{Slots}{

\describe{
\item{\code{probabilityMap}}{v-by-d numerical \code{matrix} representing the probability distribution 
temporal map (relative frequency).}

\item{\code{countsMap}}{v-by-d numerical \code{matrix} representing the counts temporal map 
(absolute frequency).}

\item{\code{dates}}{d-dimensional \code{Date} array of the temporal batches.}

\item{\code{support}}{v-by-1 numerical or character \code{matrix} representing the support 
(the value at each bin) of probabilityMap and countsMap.}

\item{\code{variableName}}{name of the variable (character).}

\item{\code{variableType}}{type of the variable (character) among "numeric", "character", "Date" and "factor".}

\item{\code{period}}{batching period among "week", "month" and "year".}
}}

\examples{

# Generation through estimateDataTemporalMap function:
dataset <- read.csv2(xzfile(system.file("extdata",
                                   "nhdsSubset.csv.xz",
                                   package="EHRtemporalVariability")), 
                     sep  = ",",
                     header = TRUE, 
                     na.strings = "", 
                     colClasses = c( "character", "numeric", "factor",
                                     "numeric" , rep( "factor", 22 ) ) )

datasetFormatted <- EHRtemporalVariability::formatDate(
                     input         = dataset,
                     dateColumn    = "date",
                     dateFormat = "\%y/\%m")

probMaps <- estimateDataTemporalMap(data = datasetFormatted, 
                     dateColumnName = "date", 
                     period         = "month")

class( probMaps[[1]] ) 

# Manual generation:
countsMatrix <- matrix(sample.int(25, size = 12*10, replace = TRUE), nrow = 12, ncol = 10)
probabilityMatrix <- sweep(countsMatrix,1,rowSums(countsMatrix),"/")
dates <- seq(Sys.Date(),(Sys.Date()+30*11),30)
x <- new('DataTemporalMap', probabilityMap = probabilityMatrix, 
                     countsMap = countsMatrix, dates = dates, support = data.frame(1:10), 
                     variableName = "example", variableType = "numeric", period = "month")
plotDataTemporalMap(x)

}
